*       PROGRAM AUTHOR:         RICHARD E. CHANDLER
******************************************************************************
*                               (email: richard@stats.ucl.ac.uk)
*       AFFILIATION:            DEPARTMENT OF STATISTICAL SCIENCE
*                                UNIVERSITY COLLEGE LONDON
*                                 GOWER STREET
*                                  LONDON WC1E 6BT
*                                   UK
******************************************************************************
*  This file contains a collection of FORTRAN routines used in all of the
*  programs relating to Generalised Linear Modelling of daily climate
*  data. For more details of these programs, see the headers to the 
*  files fit_logi.f, fit_gamm.f and simrain.f
*
*  REVISION HISTORY: see manual.
*   UCL 2002
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE FILCHK(FILNM,CHAN,STFLAG)
*
*	For opening files. We open file FILNM and check that it is either
*	OLD (if STFLAG = 1) or NEW (STFLAG=2). If it should exist and 
*	doesn't, we bail out, otherwise prompt for confirmation. The file
*	is connected to unit CHAN.
*
      INTEGER STFLAG,CHAN
      CHARACTER FILNM*12,OPTION*1

      IF (STFLAG.EQ.1) THEN
       OPEN(CHAN,FILE=FILNM,STATUS='OLD',ERR=910)
      ELSEIF (STFLAG.EQ.2) THEN
       OPEN(CHAN,FILE=FILNM,STATUS='NEW',ERR=920)
      ELSE
       WRITE(*,3) STFLAG,FILNM
       STOP
      ENDIF

      RETURN
*
*	Error trapping
*
 910  CALL READERR(FILNM,0,1)

 920  WRITE(*,2) FILNM
      READ(*,'(A1)') OPTION
      IF ( (OPTION.EQ.'Y').OR.(OPTION.EQ.'y') ) THEN
       OPEN(CHAN,FILE=FILNM)
       RETURN
      ENDIF
      STOP

 2    FORMAT(/5X,'****WARNING**** Output file ',A12,' already ',
     +'exists.',/5X,'Overwrite it (Y/N, default N)?')
 3    FORMAT(/5X,'****ERROR**** ',I2,' is an invalid value for ',
     +'STFLAG in routine',/5X,'FILCHK. Contact REC (error occurred ',
     +'while trying to',/5X,'open file ',A12)
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE MDLSET(FILNO,FILNM,MXP,MDLTYP,SIM,NATTR,
     +  NP,GLBVAL,BETA,THETA,NU,COVCODE,GLBCOD,MXLAG,SPMOD,IEXTD,
     +  IEXTM,IEXTY,SITXFM,WTSCHM,TWO,THREE,NLSTAT,FOUIDX,LEGIDX,
     +  PWTIDX,SITINF,MXN,NSITES,RHO,TITLE)
******************************************************************************
*       To read a model specification, in standard format, from file
*       FILNM attached to unit FILNO. All variables except FILNO, 
*	FILNM, SIM, MXP, MDLTYP and NATTR are output
******************************************************************************
*     	PARAMETERs
*     	^^^^^^^^^^
*	MXCPNT	- Max. no. of separate components in a model (i.e. site,
*		  year, nonlinear, spatial etc. effects). Used in 
*		  dimensioning NCODES
******************************************************************************
      INTEGER MXCPNT
      PARAMETER (MXCPNT=10)
******************************************************************************
*       INTEGER variables
*       ^^^^^^^^^^^^^^^^^
*	FILNO	- Channel number to read
*       MXP     - Maximum allowable number of parameters (used for
*                 dimensioning arrays)
*	MXN	- Maximum number of sites (ditto)
*	NSITES	- Actual number of sites. Used to check orthogonal
*		  series representations.
*	MDLTYP	- Indicates the type of the response variable - 1 for 
*		  amounts, 2 for occurrences. Used to check spatial
*		  structure (input).
*	SIM	- 1 if we're simulating & need a dispersion parameter (e.g.
*		  for intensity model) 0 otherwise (input). Also used
*		  to control whether or not we'll set derivatives
*		  of nonlinear parameters.
*       NATTR   - Number of site attributes defined (used for checking)
*       NP      - Numbers of parameters in various components of the model.
*                 See main program header for more details
*       COVCODE - Code associated with each covariate - used to define
*                 transformations etc.
*       GLBCOD  - Codes defining `global' quantities in GLBVAL
*	MXLAG	- Number of previous days' data which we require to be
*		  present at each site for inclusion in the dataset.
*	NLSTAT	- Indicator for whether or not we want to estimate nonlinear
*		  parameters or take them as known
*	SITXFM	- indicates which nonlinear functions of site properties
*		  have been selected. 
*       TWO     - Indicates two-way interactions
*       THREE   - Indicates three-way interactions
*	FOUIDX	- Indexing for Fourier representation of site effects
*	LEGIDX	- Indexing for Legendre polynomial representation of 
*		  site effects
*       PWTIDX  - Indexing for weighted averages of previous days' values
*                 (see main program files for details)
*       WTSCHM  - Indexing for weighted average of previous days' values
*                 (Ith element is the weighting scheme used in Ith
*                 covariate).
*	SPMOD	- Indicates choice of model for correlation structure
*       TRCREQ  - Is a trace threshold required?
*       IEXTD } - Indicate whether `external' predictors at daily,
*       IEXTM }   monthly or yearly timescales have been requested
*       IEXTY }
*       N       - Number of parameters in model so far. Also used to identify 
*		  lines in the input file - hence start at -2
*       COMPNT  - Indicates which component of the model this record's to do 
*                 with (i.e. how far along NP we are)
*       OLDCMP  - Value of COMPNT on previous step - used to check that
*                 input file is in the right order
*       NHEAD   - No. of header lines in file
*       CODE?   - Either to go into COVCODE or to define interactions
*	NCODES	- Minimum number of codes required for each component type
*		  (used to check input)
******************************************************************************
      INTEGER FILNO,MXP,MDLTYP,SIM,NATTR
      INTEGER NP(10),COVCODE(MXP),GLBCOD(MXP),NLSTAT(MXP,3)
      INTEGER IEXTD,IEXTM,IEXTY
      INTEGER MXLAG,SPMOD,SITXFM(MXP,2),TWO(MXP,2),THREE(MXP,3)
      INTEGER FOUIDX(MXP),LEGIDX(MXP),MXN,NSITES
      INTEGER PWTIDX(MXP,0:3),WTSCHM(MXP),TRCREQ
      INTEGER N,COMPNT,CODE(3),OLDCMP,NHEAD,NCODES(MXCPNT)
      INTEGER I
******************************************************************************
*       DOUBLE PRECISION variables
*       ^^^^^^^^^^^^^^^^^^^^^^^^^^
*       GLBVAL  - values of global quantities
*       BETA    - parameter vector for the model
*	THETA	- parameters in nonlinear functions
*	NU	- dispersion parameter
*	RHO	- parameters in specification of correlation structure
*	SITINF	- array of site information (used to check orthogonal
*		  series representations)
*       COEFF   - value as read from the input file
******************************************************************************
      DOUBLE PRECISION GLBVAL(MXP),BETA(0:MXP),THETA(MXP,3)
      DOUBLE PRECISION NU,RHO(MXP),SITINF(MXN,MXP,0:3)
      DOUBLE PRECISION COEFF
******************************************************************************
*       CHARACTER variables
*       ^^^^^^^^^^^^^^^^^^^
*       FILNM   - Input file name
*       TITLE   - Title of model
******************************************************************************
      CHARACTER FILNM*12,TITLE*70

      DATA (NCODES(I),I=1,MXCPNT) /1,1,1,1,2,3,2,1,0,1/

      N = -2
      OLDCMP = -1
      NHEAD = 46
      TRCREQ = 0
      DO 9 I=1,10
       NP(I) = 0
 9    CONTINUE
      
*
*       Skip over file header (which is NHEAD lines long)
*
      DO 10 I = 1,NHEAD
       READ(FILNO,*)
 10   CONTINUE
*
*	This line is currently redundant - will be used to choose
*       distribution in future.
*
      READ(FILNO,*,ERR=96)
      N = N + 1
*   
*       Read model description
*
      READ(FILNO,'(A70)',ERR=96) TITLE

 20   READ(FILNO,1,END=90,ERR=96) COMPNT,COEFF,(CODE(I),I=1,3)
*
*       Successful read - increment no. of parameters (started at -2 so
*       first will be zero); check that available storage is sufficient.
*
      N = N + 1
      IF (N.GT.MXP) CALL READERR(FILNM,MXP,40)

*
*       Update according to the value of COMPNT - if it's zero then we've
*       got the constant and there's nothing more to do, otherwise need
*       to check input, evaluate CODE and update NP. 
*
      IF (COMPNT.LE.6) BETA(N) = COEFF

      IF (COMPNT.EQ.0) THEN 
       IF (N.EQ.0) THEN
        GOTO 20
       ELSE
        CALL READERR(FILNM,N+3+NHEAD,2)
       ENDIF
      ENDIF
*       
*       Don't allow users to get their input file in the wrong order
*       (it will almost inevitably screw up the interactions). Also
*	check that the value of COMPNT is valid.
*
      IF (COMPNT.LT.OLDCMP) CALL READERR(FILNM,N+1+NHEAD,3)
      IF (COMPNT.GT.MXCPNT) CALL READERR(FILNM,N+1+NHEAD,20)
*
*	Now check that at least the minimum required amount of 
*	information has been given. The only situations in which a 
*	CODE of zero is allowed in a field that would normally be
*	required is if we have the default spatial structure, or if
*	we're defining the dispersion parameter. The IFs here are a bit
*	inelegant, but they work!
*
      IF ( (COMPNT.LT.9).OR.((CODE(1).NE.0).AND.(COMPNT.NE.9)) ) THEN
       IF (CODE(NCODES(COMPNT)).EQ.0) CALL READERR(FILNM,N+3+NHEAD,32)
      ENDIF
      OLDCMP = COMPNT
*
*	At this stage we update everything except NP(7) - NP(10)
*	(NP(7)=0 means something later on, and NP(9) (dispersion
*	parameter), NP(8) (global quantities) and NP(10) (spatial 
*       structure) are unrelated to the rest of the NPs.
*
       DO 25 I=COMPNT,6
        NP(I) = NP(I) + 1
 25    CONTINUE

      IF (COMPNT.LE.4) THEN
*
*	Possible error trap!
*
       IF (N.LE.0) CALL READERR(FILNM,NHEAD,4)
       COVCODE(N) = CODE(1)
*
*	For site effects, check that the requested attribute has been 
*       defined. Then record if any nonlinear transformations
*	have been defined, and remember what site attributes they
*	correspond to. Also, allow estimation of nonlinear parameters
*	here.
*
       IF (COMPNT.EQ.1) THEN
        IF (CODE(1).GT.NATTR) CALL READERR(FILNM,N+3+NHEAD,17)
        IF (CODE(2).GT.0) THEN
         SITXFM(N,1) = CODE(1)
         SITXFM(N,2) = CODE(2)
         NLSTAT(N,1) = 0
         NLSTAT(N,2) = 0
        ENDIF
       ENDIF
*
*	Also allow nonlinear parameters to be estimated for trend effects,
*       and note whether there are any external predictors on an annual
*       timescale. ISIGN is to allow for the fact that the user may
*       ask for negatively lagged external predictors (i.e. future values)
*
       IF (COMPNT.EQ.2) THEN
        NLSTAT(N,1) = 0
        NLSTAT(N,2) = 0
        IF (CODE(1).GT.50) THEN
         IEXTY = 1
         COVCODE(N) = (1000*CODE(2)) + ISIGN(COVCODE(N),CODE(2))
        ENDIF
       ENDIF
*
*     Monthly effects - note whether any external ones have been 
*     requested.
*
       IF (COMPNT.EQ.3) THEN
        IF (CODE(1).GT.50) THEN
         IEXTM = 1
         COVCODE(N) = (1000*CODE(2)) + ISIGN(COVCODE(N),CODE(2))
        ENDIF
       ENDIF
*
*	Daily effects. 
*
       IF (COMPNT.EQ.4) THEN
*
*       For `previous days' effects we may want to discard cases with
*	missing values in any of the last MXLAG days even though we're
*	fitting less than MXLAG parameters, in order that models
*	with different numbers of lags can be compared. We may also 
*       have known nonlinear transformations of previous days' values -
*       code these differently e.g. COVCODE = 3002 here means xfrmation
*       3 of value 2 days ago.
*
        IF (CODE(1).LE.10) THEN
         MXLAG = MAX0(MXLAG,CODE(3),CODE(1))
         IF (MXLAG.GT.10) CALL READERR(FILNM,N+3+NHEAD,22)
         IF (CODE(2).GT.0) COVCODE(N) = COVCODE(N) + (1000*CODE(2))
*
*     Mark any request for a trace value - so that we can check that the
*     threshold gets defined.
*
         IF (MOD(CODE(2),10).EQ.4) TRCREQ = 1
*
*     Are we doing spatial averaging of previous days' values, that
*     involves nonlinear functions of inter-site distance? If so,
*     make sure we've got at least 2 site attributes defined (otherwise
*     we can't calculate distances).
*
         IF (MOD(CODE(2)/10,10).GT.1) THEN
          IF (NATTR.LT.2) CALL READERR(FILNM,N+3+NHEAD,18)
          WTSCHM(N) = MOD(CODE(2)/10,10)
          NLSTAT(N,1) = 0
          NLSTAT(N,2) = 0
	  NLSTAT(N,3) = 0
         ENDIF
*
*     Check no transformations requested for anything other than 
*     previous days' effects (may be `daily' effects other than 
*     previous days)
*
        ELSE
         IF (CODE(2).GT.0) CALL READERR(FILNM,N+3+NHEAD,31)
         IF (CODE(1).GT.50) THEN
          IEXTD = 1
          COVCODE(N) = (1000*CODE(2)) + ISIGN(COVCODE(N),CODE(2))
         ENDIF
        ENDIF
       ENDIF
*
*       2-way interactions - we check that the terms requested are in fact 
*       main effects
*
      ELSEIF (COMPNT.EQ.5) THEN
       IF ( ( .NOT.((CODE(1).GE.1).AND.(CODE(1).LE.NP(4))) ).OR.
     +      ( .NOT.((CODE(2).GE.1).AND.(CODE(2).LE.NP(4))) ) ) THEN
	CALL READERR(FILNM,N+3+NHEAD,21)
       ENDIF
       TWO(N,1) = CODE(1)
       TWO(N,2) = CODE(2)
*
*       And 3-way interactions
*
      ELSEIF (COMPNT.EQ.6) THEN
       IF ( ( .NOT.((CODE(1).GE.1).AND.(CODE(1).LE.NP(4))) ).OR.
     +      ( .NOT.((CODE(2).GE.1).AND.(CODE(2).LE.NP(4))) ).OR.
     +      ( .NOT.((CODE(3).GE.1).AND.(CODE(3).LE.NP(4))) ) ) THEN
	CALL READERR(FILNM,N+3+NHEAD,21)
       ENDIF
       THREE(N,1) = CODE(1)
       THREE(N,2) = CODE(2)
       THREE(N,3) = CODE(3)
*
*	Now parameters in nonlinear functions. CODE(1) will be the
*	predictor where the nonlinear function was requested
*	(NB we know what the function is, because it was defined
*	by the request); CODE(2) - either 1 or 2 - the number
*	of the parameter we're currently defining, and CODE(3), if
*	present and non-zero, indicates that we're supposed to estimate
*	the parameter rather than take it as known (the default). The check 
*	that THETA(CODE(1),CODE(2)) is less than 10^8 is to ensure that we
*	haven't already defined it. If the parameter is to be estimated
*	rather than assumed known, we duplicate it in BETA, as then the
*	IWLS procedure is easier to implement. Useful to have in THETA
*	as well, for when we come to recalculate nonlinear functions.
*
      ELSEIF (COMPNT.EQ.7) THEN
       IF ( ( .NOT.((CODE(1).GE.1).AND.(CODE(1).LE.NP(4))) ).OR.
     +      ( .NOT.((CODE(2).GE.1).AND.(CODE(2).LE.3)) ) ) THEN
	CALL READERR(FILNM,N+3+NHEAD,33)
       ELSEIF (THETA(CODE(1),CODE(2)).LT.1.0D8) THEN
        CALL READERR(FILNM,N+3+NHEAD,30)
       ELSEIF (NLSTAT(CODE(1),CODE(2)).NE.0) THEN
        CALL READERR(FILNM,N+3+NHEAD,31)
       ENDIF
*
*	We've found a nonlinear parameter, so NP(7) is no longer 
*	zero. For nonlinear parameters, COVCODE is used to 
*	identify (a) the covariate number with which it is
*	associated (b) the parameter number for that covariate.
*	The formula is COVCODE = 1000a + b. E.g. COVCODE = 3002
*	means covariate 3, parameter 2. We copy it to both the 
*	parameter vector (for updating estimates) and to the
*	THETA array (easier bookkeeping when calculating covariates)
*	Note that if we're estimating a nonlinear parameter, the 
*	associated BETA can't be zero. Also, parameters for 
*	orthogonal series representations of site attributes are 
*	always treated as known, so check that we haven't been 
*	asked to estimate these. There are additional complications
*	for orthogonal series and weighted averages of previous days'
*       values, like I'm only going to allow limits to be specified 
*       once.
*
       IF (NP(7).EQ.0) NP(7) = NP(6)
*
*	Orthogonal series options are fiddly and relegated to 
*	a separate subroutine.
*
       IF ( (SITXFM(CODE(1),2).GE.11).AND.
     +      (SITXFM(CODE(1),2).LE.30) ) THEN
        CALL ORSET(SITXFM,CODE,FOUIDX,THETA,SITINF,NSITES,FILNM,
     +					NHEAD,N,COEFF,MXP,MXN)
       ELSEIF ( (SITXFM(CODE(1),2).GE.31).AND.
     +          (SITXFM(CODE(1),2).LE.40) ) THEN
        CALL ORSET(SITXFM,CODE,LEGIDX,THETA,SITINF,NSITES,FILNM,
     +					NHEAD,N,COEFF,MXP,MXN)
*
*     Ditto weighted averages of previous days' values
*
       ELSEIF (WTSCHM(CODE(1)).GT.1) THEN 
        CALL PWTDEF(WTSCHM,CODE,PWTIDX,THETA,FILNM,NHEAD,
     +                                                N,COEFF,MXP)
*
*	No such complications with any other nonlinear functions
*
       ELSE
        THETA(CODE(1),CODE(2)) = COEFF
       ENDIF
*
*	Now see if the parameter is to be estimated & deal accordingly
*       (weighting schemes: copy covariate code into index for 
*       derivatives as well)
*
       IF ( (CODE(3).NE.0).AND.(SIM.NE.1) ) THEN
        NLSTAT(CODE(1),CODE(2)) = CODE(3)
        IF (DABS(BETA(CODE(1))).LE.0.0D0) THEN
         CALL READERR(FILNM,N+3+NHEAD,35)
        ENDIF
        NP(7) = NP(7) + 1
        COVCODE(NP(7)) = (1000*CODE(1)) + CODE(2)
        BETA(NP(7)) = COEFF
        IF (WTSCHM(CODE(1)).GT.1) 
     +       PWTIDX(WTSCHM(CODE(1)),CODE(2)) = NP(7)
       ENDIF
*
*     `Global' quantities. Each can be defined at most once. We've
*     already checked that CODE(1) has been defined. We also record
*     whether or not a trace threshold has been defined, by setting
*     TRCREQ=0 if it has (translates as `no longer require the 
*     threshold to be defined')
*
      ELSEIF (COMPNT.EQ.8) THEN
       NP(8) = NP(8) + 1
       GLBCOD(NP(8)) = (1000*CODE(1)) + CODE(2)
       IF (DABS(GLBVAL(CODE(1))).LE.1.0D8) 
     +                 CALL READERR(FILNM,N+3+NHEAD,30)
       IF (CODE(1).EQ.1) THEN
        IF (CODE(2).EQ.0) CALL READERR(FILNM,N+3+NHEAD,32)
        IF ((CODE(2).LT.0).OR.(CODE(2).GT.3)) 
     +                    CALL READERR(FILNM,N+3+NHEAD,34)
        IF (CODE(2).EQ.1) TRCREQ = 0
       ELSE
        CALL READERR(FILNM,N+3+NHEAD,34)
       ENDIF
       GLBVAL(CODE(1)) = COEFF
*
*	Dispersion parameter. No more than 1! (and none for a logistic
*       model)
*
      ELSEIF (COMPNT.EQ.9) THEN
       IF (NP(9).EQ.1) CALL READERR(FILNM,N+3+NHEAD,50)
       IF (MDLTYP.EQ.1) CALL READERR(FILNM,N+3+NHEAD,52)
       NP(9) = NP(9) + 1
       NU = COEFF
*
*       Here's specification of spatial structure. We check (a) that all
*	entries relating to spatial structure attempt to define the same
*	model (b) that no parameter is defined more than once (c) that the
*	spatial structure being defined is appropriate for the type of
*	model being used (variable MDLTYP). We use a 
*	similar coding for COVCODE e.g. COVCODE = 3002 means covariance 
*	model 3, parameter 2. Distinguish between `default' and `requested'
*	selection of model 0 (`requested') gets set to -1 for now)
*
      ELSEIF (COMPNT.EQ.10) THEN
       IF (SPMOD.NE.0) THEN
        IF (SPMOD.NE.CODE(1).AND.(
     +      .NOT.((SPMOD.EQ.-1).AND.(CODE(1).EQ.0)) ) ) THEN
          CALL READERR(FILNM,N+3+NHEAD,45)
        ELSEIF ( (CODE(2).EQ.0).AND.(SPMOD.GT.1) ) THEN
         CALL READERR(FILNM,N+3+NHEAD,32)        
        ENDIF
       ENDIF
       IF ( (CODE(2).NE.0).AND.(RHO(CODE(2)).LT.1.0D8) ) THEN
        CALL READERR(FILNM,N+3+NHEAD,30)
*
*	Spatial structures 1 through 20 are generic, but 21 through 
*       40 are only valid for logistic model (MDLTYP=1).
*
       ELSEIF ( ((MDLTYP.NE.1).AND.((CODE(1)-1)/20.NE.0)) ) THEN
        CALL READERR(FILNM,N+3+NHEAD,47)
       ENDIF
       IF (CODE(1).LT.1) THEN
        SPMOD = -1
       ELSEIF (CODE(1).NE.1) THEN
        IF (CODE(2).EQ.0) CALL READERR(FILNM,N+3+NHEAD,32)
        RHO(CODE(2)) = COEFF
        NP(10) = NP(10)+ 1
        SPMOD = CODE(1)
       ELSE
        SPMOD = CODE(1)
       ENDIF
*
*	Trap invalid values of COMPNT
*
      ELSE
       CALL READERR(FILNM,N+3+NHEAD,20)
      ENDIF
      GOTO 20
*
*	Reset `requested' default spatial structure to model 0, and 
*	check that a dispersion parameter has been defined if we're in
*	'simulation' mode.
*
 90   IF (SPMOD.EQ.-1) SPMOD = 0
      IF ( (NP(9).EQ.0).AND.(SIM.EQ.1) ) CALL READERR(FILNM,0,51)
*
*     Finally, check that trace threshold has been defined if it's 
*     required
*
      IF (TRCREQ.EQ.1) CALL READERR(FILNM,0,25)

      RETURN
*
*       Error trapping
*
 96   CALL READERR(FILNM,N+3+NHEAD,2)

 1    FORMAT(I5,F10.4,3I5)
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE ORSET(SITXFM,CODE,ORTIDX,THETA,SITINF,NSITES,FILNM,
     +					NHEAD,N,COEFF,MXP,MXN)
*
*	To set limits for orthogonal series representations of site
*	effects, and check that these have been defined correctly.
*	This stuff really belongs in the MDLSET routine, but it's so
*	fiddly that it's easier to read if relegated to its own
*	personal space.
******************************************************************************
*	ARGUMENTS
*	^^^^^^^^^
*	SITXFM	- indicates which nonlinear functions of site properties
*		  have been selected. INTEGER, input.
*       CODE    - Identifiers of attribute being defined, read from input
*		  file. INTEGER, input.
*	ORTIDX	- Index array to track attributes for which orthogonal
*		  series representations are being defined. INTEGER,
*		  input/output.
*	THETA	- parameters in nonlinear functions. In this routine, 
*		  elements of THETA get set to limits for orthogonal
*		  series representation. DOUBLE PRECISION, input/output.
*	SITINF	- Array of site information, used to check validity of
*		  limits. DOUBLE PRECISION, input.
*	NSITES	- Number of sites used. INTEGER, input.
*       FILNM   - Input file name. Used for identifying where errors are
*		  found.  CHARACTER, input.
*       NHEAD   - No. of header lines in file. INTEGER, input.
*	N	- Used to identify whereabouts in the input file we are,
*		  for error tracking. INTEGER, input.
*       COEFF   - value we're currently trying to allocate. DOUBLE
*		  PRECISION, input.
*       MXP     - Maximum allowable number of parameters (used for
*                 dimensioning arrays). INTEGER, input.
*	MXN	- Used in dimensioning SITINF.
******************************************************************************
      INTEGER MXP,MXN,SITXFM(MXP,2),ORTIDX(MXP),CODE(3)
      INTEGER NHEAD,N,NSITES
      DOUBLE PRECISION THETA(MXP,3),COEFF,SITINF(MXN,MXP,0:3)
      CHARACTER FILNM*12
******************************************************************************
*	Additional INTEGERs
*	^^^^^^^^^^^^^^^^^^^
*	OSET	- Flag to keep track of where we're at when defining
*		  orthogonal series representations
*	I	- Counter
******************************************************************************
      INTEGER OSET,I
******************************************************************************
*	Additional DOUBLES
*	^^^^^^^^^^^^^^^^^^
*	LL	- Lower limit for representation
*	UL	- Upper limit
******************************************************************************
      DOUBLE PRECISION LL,UL

*
*	First check: all parameters here should be regarded as known
*
      IF (CODE(3).NE.0) CALL READERR (FILNM,N+3+NHEAD,36)
*
*	Next: has this limit been defined somewhere else?
*
      IF (ORTIDX(SITXFM(CODE(1),1)).NE.0) THEN
       IF ( (THETA(ORTIDX(SITXFM(CODE(1),1)),1).LT.1.0D8).AND.
     +      (THETA(ORTIDX(SITXFM(CODE(1),1)),2).LT.1.0D8) )
     +        			CALL READERR (FILNM,N+3+NHEAD,30)
      ENDIF
*
*	If we've got here, things are looking basically OK so set the 
*	parameter: if it's the first one being defined for this attribute,
*	record it.
*
      IF ( ORTIDX(SITXFM(CODE(1),1)).EQ.0) THEN
       ORTIDX(SITXFM(CODE(1),1)) = CODE(1)
       OSET = 1
      ELSE
       OSET = 2
      ENDIF
      THETA(ORTIDX(SITXFM(CODE(1),1)),CODE(2)) = COEFF
*
*	If both parameters have been defined, some further checks ...
*
      IF ( OSET.EQ.2 ) THEN
*
*	Are they in the right order? (don't allow UL < LL as it may 
*	indicate some other inadvertent cock-up). 
*
       LL = THETA(ORTIDX(SITXFM(CODE(1),1)),1)
       UL = THETA(ORTIDX(SITXFM(CODE(1),1)),2)
       IF ( LL.GE.UL ) CALL READERR(FILNM,N+3+NHEAD,38)
*
*	Are all sites within the range? If not, representation is 
*	invalid.
*
       DO 100 I=1,NSITES
        IF ( (SITINF(I,SITXFM(CODE(1),1),0).LE.LL). OR. 
     +       (SITINF(I,SITXFM(CODE(1),1),0).GE.UL) ) THEN
         CALL READERR(FILNM,I,39)
        ENDIF
 100   CONTINUE
      ENDIF

      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE PWTDEF(WTSCHM,CODE,PWTIDX,THETA,FILNM,
     +					NHEAD,N,COEFF,MXP)
*
*	To define parametrisation of weighted averages of previous days'
*	values, and check that these have been defined correctly. Same
*       idea as ORSET routine above.
******************************************************************************
*	ARGUMENTS
*	^^^^^^^^^
*	WTSCHM	- indicates which weighting scheme has been selected.
*		  INTEGER, input.
*       CODE    - Identifiers of attribute being defined, read from input
*		  file. INTEGER, input.
*	PWTIDX	- Index array to track predictors for which weighting
*		  parameters are being defined. INTEGER, input/output.
*	THETA	- parameters in nonlinear weighting functions. 
*                 DOUBLE PRECISION, input/output.
*       FILNM   - Input file name. Used for identifying where errors are
*		  found.  CHARACTER, input.
*       NHEAD   - No. of header lines in file. INTEGER, input.
*	N	- Used to identify whereabouts in the input file we are,
*		  for error tracking. INTEGER, input.
*       COEFF   - value we're currently trying to allocate. DOUBLE
*		  PRECISION, input.
*       MXP     - Maximum allowable number of parameters (used for
*                 dimensioning arrays). INTEGER, input.
******************************************************************************
      INTEGER MXP,WTSCHM(MXP),PWTIDX(MXP,0:3),CODE(3)
      INTEGER NHEAD,N
      DOUBLE PRECISION THETA(MXP,3),COEFF
      CHARACTER FILNM*12

*
*	Next: has this limit been defined somewhere else?
*
      IF (PWTIDX(WTSCHM(CODE(1)),0).NE.0) THEN
       IF ( (THETA(PWTIDX(WTSCHM(CODE(1)),0),CODE(2)).LT.1.0D8) )
     +        			CALL READERR (FILNM,N+3+NHEAD,30)
      ENDIF
*
*	If we've got here, things are looking basically OK so set the 
*	parameter: if it's the first one being defined for this attribute,
*	record the covariate to which it's attached. Otherwise make
*       a note of the model parameter number.
*
      IF ( PWTIDX(WTSCHM(CODE(1)),0).EQ.0) THEN
       PWTIDX(WTSCHM(CODE(1)),0) = CODE(1)
      ENDIF
      THETA(PWTIDX(WTSCHM(CODE(1)),0),CODE(2)) = COEFF

      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE TXTSET(NP,COVCODE,GLBCOD,SITXFM,PWTIDX,WTSCHM,
     +     ATTRTXT,SXPTXT,TRNDTXT,TRPTXT,MOTXT,DYTXT,PWTTXT,MDLTXT,
     +     NLTXT,GLBTXT,MXP)
*
*       Sets up descriptive text for the model whose covariate
*       structure and codes are held in NP and COVCODE. Variables:
*       NP      - array indicating how many elements are associated
*                 with each model component. See main program header.
*                 Input.
*       COVCODE - Model specification for each covariate. Input
*       GLBCOD  - Codes for `global' quantities. Input
*	SITXFM	- Chooses nonlinear transformations of site attribs. Input
*       PWTIDX  - Indexing for weighted averages of previous days
*       WTSCHM  - Chooses weighting scheme for averaging previous days'
*                 values. Input.
*       ATTRTXT - Descriptive text for site attributes. Input
*       TRNDTXT - Descriptive text for trend functions. Input
*	TRPTXT	- Describes parameters in trend fns. Input
*       MOTXT   - Descriptive text for monthly functions. Input
*       DYTXT   - Descriptive text for previous days' dependence. Input
*       MDLTXT  - Text corresponding to each element of chosen model.
*                 Output.
*	NLTXT	- Text describing nonlinear parameters in model. Output.
*       MXP     - Max. number of model parameters. Input
*****************************************************************************
      INTEGER NP(10),MXP
      INTEGER COVCODE(MXP),SITXFM(MXP,2),WTSCHM(MXP)
      INTEGER GLBCOD(MXP),PWTIDX(MXP,0:3)
      CHARACTER ATTRTXT(MXP)*70,TRNDTXT(MXP)*70
      CHARACTER*70 MOTXT(MXP),DYTXT(MXP),PWTTXT(MXP,3)
      CHARACTER MDLTXT(0:MXP)*70
      CHARACTER TRPTXT(MXP,3)*70,NLTXT(MXP,3)*70
      CHARACTER SXPTXT(MXP,3)*70,GLBTXT(MXP)*70
*****************************************************************************
*	Additional INTEGERs
*	^^^^^^^^^^^^^^^^^^^
*	NTRND	- Number of current trend function
* 	I	- counter
*****************************************************************************
      INTEGER NTRND,I
*****************************************************************************
*	Additional CHARACTERs
*	^^^^^^^^^^^^^^^^^^^^^
*	TMPCH?	- Temporary storage
*****************************************************************************
      CHARACTER TMPCH1*11,TMPCH2*70

      MDLTXT(0) = 'Constant'
      
      DO 10 I=1,NP(1)
       MDLTXT(I) = ATTRTXT(COVCODE(I))
       IF (SITXFM(I,1).NE.0) THEN
        NLTXT(I,1) = SXPTXT(SITXFM(I,2),1)
        NLTXT(I,2) = SXPTXT(SITXFM(I,2),2)
        NLTXT(I,3) = SXPTXT(SITXFM(I,2),3)
       ENDIF
 10   CONTINUE
      
      DO 20 I=NP(1)+1,NP(2)
*
*	For trend functions, we allow the possibility of more than one
*	and identify them with the prefix `Trend 1' etc.
*     
       IF (MOD(IABS(COVCODE(I)),1000).LE.50) THEN
        NTRND = I - NP(1)
        WRITE(TMPCH1,'(''Trend '',I2,'' - '')') NTRND
        TMPCH2 = TRNDTXT(COVCODE(I))
        MDLTXT(I) = TMPCH1//TMPCH2(1:59)
        NLTXT(I,1) = TRPTXT(COVCODE(I),1)
        NLTXT(I,2) = TRPTXT(COVCODE(I),2)
        NLTXT(I,3) = TRPTXT(COVCODE(I),3)
*
*     Set text for `external' annual influences if necessary
*
       ELSE
        CALL EXTXST(31,1,MOD(IABS(COVCODE(I)),1000)-50,
     +                                 COVCODE(I)/1000,MDLTXT(I))
       ENDIF
 20   CONTINUE

*
*     Monthly ...
*
      DO 30 I=NP(2)+1,NP(3)
       IF (MOD(IABS(COVCODE(I)),1000).LE.50) THEN
        MDLTXT(I) = MOTXT(COVCODE(I))
       ELSE
        CALL EXTXST(32,2,MOD(IABS(COVCODE(I)),1000)-50,
     +                                 COVCODE(I)/1000,MDLTXT(I))
       ENDIF
 30   CONTINUE
*
*     Daily effects. For those involving previous days' values,
*     we may have to change the existing text to process nonlinear
*     transformations.
*
      DO 40 I=NP(3)+1,NP(4)
       IF (MOD(IABS(COVCODE(I)),1000).LE.50) THEN
        MDLTXT(I) = DYTXT(MOD(COVCODE(I),1000))
        IF (COVCODE(I).GT.1000) THEN
         CALL PRVTXT(COVCODE(I),MDLTXT(I))
         IF ((WTSCHM(I).GT.1).AND.(PWTIDX(WTSCHM(I),0).EQ.I)) THEN
          IF (WTSCHM(I).NE.3) THEN
           NLTXT(I,1) = PWTTXT(WTSCHM(I),1)
           NLTXT(I,2) = PWTTXT(WTSCHM(I),2)
          ELSE
           NLTXT(I,1) = 'Offset for '//ATTRTXT(1)
           NLTXT(I,2) = 'Offset for '//ATTRTXT(2)
          ENDIF
          NLTXT(I,3) = PWTTXT(WTSCHM(I),3)
*         ELSE
*          IF(MOD(COVCODE(I)/1000,10).EQ.4) THEN
*           NLTXT(I,1) = 'Trace threshold'
*          ENDIF
         ENDIF
        ENDIF
       ELSE
        CALL EXTXST(33,3,MOD(IABS(COVCODE(I)),1000)-50,
     +                                 COVCODE(I)/1000,MDLTXT(I))
       ENDIF
 40   CONTINUE

*
*     `Global' quantities. No need to trap `undefined' quantities
*     here, as this has already been done in MDLSET.
*
      DO 80 I=1,NP(8)
       IF (GLBCOD(I)/1000.EQ.1) THEN
        IF (MOD(GLBCOD(I),1000).EQ.1) THEN
         GLBTXT(I) = 'Trace threshold'
        ELSEIF (MOD(GLBCOD(I),1000).EQ.2) THEN
         GLBTXT(I) = '`Soft'' threshold for +ve values'
        ELSEIF (MOD(GLBCOD(I),1000).EQ.3) THEN
         GLBTXT(I) = '`Hard'' threshold for +ve values'
        ENDIF
       ENDIF
 80   CONTINUE

      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE CHKMDL(BETA,SEBETA,THETA,SETHETA,GLBVAL,NU,RHO,
     +   COVCODE,TWO,THREE,GLBCOD,NP,NLSTAT,PWTIDX,SPMOD,TITLE,
     +   MDLTXT,NLTXT,SPATXT,SPPTXT,CORTXT,ATTRTXT,GLBTXT,CNVRGE,
     +   MDLTYP,MXP)
*
*       To output current model parametrisation
*
*       INTEGER variables
*       ^^^^^^^^^^^^^^^^^
*       MXP     - Maximum allowable number of parameters (used for
*                 dimensioning arrays)
*       MDLTYP  - Code indicating which type of model we're dealing with:
*                 1 for a logistic regression, 11 for a gamma GLM
*       NP      - Numbers of parameters in various components of the model.
*                 See main program header for more details
*       COVCODE - Code assiciated with each covariate - used to define
*                 transformations etc.
*       TWO     - Indicates two-way interactions
*       THREE   - Indicates three-way interactions
*       GLBCOD  - Defines global quantities in GLBVAL
*	NLSTAT	- Indicates whether nonlinear parameters are known or to
*		  be estimated
*       PWTIDX  - Index telling us which main effect to associate
*                 with parameters in weighting schemes.
*	SPMOD	- Spatial correlation model
*	SPPARM	- Number of spatial parameter we're currently dealing with
*       LNTITL  - Length of TITLE string
*       GLOBID  - Identifier for the `global' quantity currently being
*                 considered
*	CNVRGE	- indicates convergence status
*       PWTFLG  - Flag indicating whether there's any distance-weighted
*                 averaging of previous days' values going on.
*       WTSCHM  - Weighting scheme used.
*       XFRM    - Underlying transformation of previous values
******************************************************************************
      INTEGER MXP,NP(10),COVCODE(MXP),TWO(MXP,2),THREE(MXP,3)
      INTEGER GLBCOD(MXP),NLSTAT(MXP,3),PWTIDX(MXP,0:3),SPMOD
      INTEGER LNTITL,GLOBID,CNVRGE,MDLTYP
      INTEGER I,J,PWTFLG,WTSCHM,XFRM
******************************************************************************
*       DOUBLE PRECISION variables
*       ^^^^^^^^^^^^^^^^^^^^^^^^^^
*       BETA    - parameter vector for the model
*	SEBETA	- vector of standard errors (only used on final pass)
*	THETA	- parameters in nonlinear transformations
*	SETHETA	- standard errors of THETA
*       GLBVAL  - Values of global quantities
*	NU	- dispersion parameter in model. A negative value
*		  is interpreted as meaning that the model doesn't
*		  require a dispersion parameter.
*	RHO	- parameters in spatial model
******************************************************************************
      DOUBLE PRECISION BETA(0:MXP),SEBETA(0:MXP),GLBVAL(MXP)
      DOUBLE PRECISION THETA(MXP,3),SETHETA(MXP,3),NU,RHO(MXP)
******************************************************************************
*       CHARACTER variables
*       ^^^^^^^^^^^^^^^^^^^
*       MDLTXT  - array containing descriptive text for each covariate
*	NLTXT	- array containing descriptive text for nonlinear parameters
*	SPATXT  - array containing descriptive text for spatial model
*	SPPTXT	- array containing descriptive text for spatial paramaters
*       ATTRTXT - array containing descriptive text for site attributes
*       GLBTXT  - array containing descriptive text for global quantities
*       CORTXT  - array describing quantities to which inter-site 
*                 correlations correspond
*       SPDESC  - Description of the spatial dependence structure
*       TITLE   - Title of model
******************************************************************************
      CHARACTER*70 MDLTXT(0:MXP),NLTXT(MXP,3),TITLE
      CHARACTER*70 SPATXT(0:MXP),SPPTXT(MXP,4),CORTXT(MXP)
      CHARACTER*70 ATTRTXT(MXP),GLBTXT(MXP),SPDESC
*
*	If this is 'initialisation' pass, print model title.
*       Find out how long the title text actually is
*
      PWTFLG = 0
      IF (CNVRGE.EQ.1) THEN
       LNTITL = INDEX(TITLE,'  ')
       IF (LNTITL.EQ.0) LNTITL = 71
       LNTITL = LNTITL-1

       WRITE(*,'(A70,/,70(A1))') TITLE,('=',I=1,LNTITL)
       WRITE(60,'(A70,/,70(A1))') TITLE,('=',I=1,LNTITL)

       WRITE(*,'(''Initial parameter estimates:'')')
       WRITE(60,'(''Initial parameter estimates:'')')
      ELSEIF (CNVRGE.EQ.3) THEN
       WRITE(*,'(''Final parameter estimates:'')')
       WRITE(60,'(''Final parameter estimates:'')')
      ENDIF
*
*	Output parameter estimates: initial and final ones to screen and
*	file, intervening ones to screen only. Also, only output standard
*	errors after final iteration
*  
      IF (CNVRGE.NE.3) THEN
       WRITE(*,'(/,''Main effect:'',T56,''Coefficient'')')
       WRITE(*,'(12(''-''),T56,11(''-''))')
       IF (CNVRGE.EQ.1) THEN
        WRITE(60,'(/,''Main effect:'',T56,''Coefficient'')')
        WRITE(60,'(12(''-''),T56,11(''-''))')
       ENDIF
      ELSE
       WRITE(*,'(/,''Main effect:'',T56,''Coefficient  Std Err'')')
       WRITE(*,'(12(''-''),T56,11(''-''),2X,7(''-''))')
       WRITE(60,'(/,''Main effect:'',T56,''Coefficient  Std Err'')')
       WRITE(60,'(12(''-''),T56,11(''-''),2X,7(''-''))')
      ENDIF

      DO 100 I = 0,NP(4)
       IF (CNVRGE.NE.3) THEN
        WRITE(*,'(A53,T56,F10.6)') MDLTXT(I),BETA(I)
        IF (CNVRGE.EQ.1) THEN 
         WRITE(60,'(A53,T56,F10.6)') MDLTXT(I),BETA(I)
        ENDIF
       ELSE
        WRITE(*,'(A53,T56,F10.6,2X,F7.4)') MDLTXT(I),BETA(I),SEBETA(I)
        WRITE(60,'(A53,T56,F10.6,2X,F7.4)') MDLTXT(I),BETA(I),SEBETA(I)
       ENDIF
 100  CONTINUE

      IF (NP(5).GT.NP(4)) THEN
       IF (CNVRGE.NE.3) THEN
        WRITE(*,'(/,''2-way interactions:'',T56,''Coefficient'')')
        WRITE(*,'(19(''-''),T56,11(''-''))')
        IF (CNVRGE.EQ.1) THEN
         WRITE(60,'(/,''2-way interactions:'',T56,''Coefficient'')')
         WRITE(60,'(19(''-''),T56,11(''-''))')
        ENDIF
       ELSE
        WRITE(*,'(/,A19,T56,A11,2X,A7)') '2-way interactions:',
     + 					  'Coefficient','Std Err'
        WRITE(*,'(19(''-''),T56,11(''-''),2X,7(''-''))')
        WRITE(60,'(/,A19,T56,A11,2X,A7)') '2-way interactions:',
     + 					  'Coefficient','Std Err'
        WRITE(60,'(19(''-''),T56,11(''-''),2X,7(''-''))')
       ENDIF
      
       DO 150 I = NP(4)+1,NP(5)
        IF (CNVRGE.NE.3) THEN
	 WRITE(*,'(A53,T56,F10.6,/''with '',A40)') 
     +          MDLTXT(TWO(I,1)),BETA(I),MDLTXT(TWO(I,2))
	 IF (CNVRGE.EQ.1) THEN
          WRITE(60,'(A40,T56,F10.6,/''with '',A40)') 
     +          	  MDLTXT(TWO(I,1)),BETA(I),MDLTXT(TWO(I,2))
         ENDIF
	ELSE
         WRITE(*,'(A53,T56,F10.6,2X,F7.4,/''with '',A40)') 
     +         	MDLTXT(TWO(I,1)),BETA(I),SEBETA(I),MDLTXT(TWO(I,2))
         WRITE(60,'(A53,T56,F10.6,2X,F7.4,/''with '',A40)') 
     +         	MDLTXT(TWO(I,1)),BETA(I),SEBETA(I),MDLTXT(TWO(I,2))
        ENDIF
 150   CONTINUE
      ENDIF
      
      IF (NP(6).GT.NP(5)) THEN
       IF (CNVRGE.NE.3) THEN
        WRITE(*,'(/,''3-way interactions:'',T56,''Coefficient'')')
        WRITE(*,'(19(''-''),T56,11(''-''))')
        IF (CNVRGE.EQ.1) THEN
         WRITE(60,'(/,''3-way interactions:'',T56,''Coefficient'')')
         WRITE(60,'(19(''-''),T56,11(''-''))')
        ENDIF
       ELSE
        WRITE(*,'(/,A19,T56,A11,2X,A7)') '3-way interactions:',
     + 					  'Coefficient','Std Err'
        WRITE(*,'(19(''-''),T56,11(''-''),2X,7(''-''))')
        WRITE(60,'(/,A19,T56,A11,2X,A7)') '3-way interactions:',
     + 					  'Coefficient','Std Err'
        WRITE(60,'(19(''-''),T56,11(''-''),2X,7(''-''))')
       ENDIF
       DO 160 I = NP(5)+1,NP(6)
        IF (CNVRGE.NE.3) THEN
	 WRITE(*,'(A53,T56,F10.6,/''with '',A40,/''and '',A40)') 
     +    MDLTXT(THREE(I,1)),BETA(I),MDLTXT(THREE(I,2)),
     +    MDLTXT(THREE(I,3))
	 IF (CNVRGE.EQ.1) THEN
          WRITE(60,'(A53,T56,F10.6,/''with '',A40,/''and '',A40)') 
     +    MDLTXT(THREE(I,1)),BETA(I),MDLTXT(THREE(I,2)),
     +    MDLTXT(THREE(I,3))
         ENDIF
	ELSE
         WRITE(*,'(A53,T56,F10.6,2X,F7.4,/A5,A40,/A4,A40)') 
     +   MDLTXT(THREE(I,1)),BETA(I),SEBETA(I),
     +	 'with ',MDLTXT(THREE(I,2)),'and ',MDLTXT(THREE(I,3))
         WRITE(60,'(A53,T56,F10.6,2X,F7.4,/A5,A40,/A4,A40)') 
     +   MDLTXT(THREE(I,1)),BETA(I),SEBETA(I),
     +	 'with ',MDLTXT(THREE(I,2)),'and ',MDLTXT(THREE(I,3))
        ENDIF
 160   CONTINUE
      ENDIF
*
*	We're set up so that if there are *no* nonlinear parameters
*	at all in the model, NP(7) is zero; if there *are* nonlinear
*	parameters but they're treated as known, NP(7)=NP(6), but we
*	still need to say something about them!
*
      IF ( (NP(7).GE.NP(6)).AND.(NP(6).NE.0)) THEN
       WRITE(*,'(/,''Parameters in non-linear transformations:'')')
       WRITE(*,'(40(''-''))')
       IF (CNVRGE.NE.2) THEN
        WRITE(60,'(/,''Parameters in non-linear transformations:'')')
        WRITE(60,'(40(''-''))')
       ENDIF
*
*	For nonlinear parameters, it's easier to go through the covariates
*	themselves looking for nonlinear parameters - otherwise we end up
*	doubling the output if there are 2 parameters if we're not
*	careful! The way to find out is if a value for the first THETA has
*	been specified. The additional IF is in case there isn't a second
*	THETA. We also flag whether or not there's a parameter associated 
*       with a distance-weighted average of previous days' values - if
*       so we'll add a footnote to tell the user where the distances are 
*       coming from.
*
       DO 170 I = 1,NP(4)
        IF (THETA(I,1).LT.1.0D8) THEN
         WTSCHM = MOD(COVCODE(I)/10000,10)
         XFRM = MOD(COVCODE(I)/1000,10)
         IF (I.GT.NP(3)) THEN
          IF (WTSCHM.GT.1) PWTFLG = 1
          IF (I.NE.PWTIDX(WTSCHM,0)) GOTO 170
         ENDIF
         WRITE(*,'(A53,'':'')') MDLTXT(I)
         IF (CNVRGE.NE.2) WRITE(60,'(A53,'':'')') MDLTXT(I)
         DO 175 J=1,3
          IF (THETA(I,J).LT.1.0D8) THEN
           IF (NLSTAT(I,J).EQ.0) THEN
            WRITE(*,'(10X,A30,'':'',1X,F9.4,1X,''(prespecified)'')')
     +          				NLTXT(I,J),THETA(I,J)
            IF (CNVRGE.NE.2) THEN
             WRITE(60,'(10X,A30,'':'',1X,F9.4,1X,''(prespecified)'')')
     +          				NLTXT(I,J),THETA(I,J)
            ENDIF
           ELSE
            IF (CNVRGE.NE.3) THEN
             WRITE(*,'(10X,A30,'':'',1X,F9.4,1X)')
     +          			NLTXT(I,J),THETA(I,J)
             IF (CNVRGE.EQ.1) WRITE(60,'(10X,A30,'':'',1X,F9.4,1X)')
     +          				NLTXT(I,J),THETA(I,J)
            ELSE
             WRITE(*,'(10X,A30,A2,F9.4,1X,A10,F9.4,A1)')
     +        NLTXT(I,J),': ',THETA(I,J),'(Std Err: ',SETHETA(I,J),')'
             WRITE(60,'(10X,A30,A2,F9.4,1X,A10,F9.4,A1)')
     +        NLTXT(I,J),': ',THETA(I,J),'(Std Err: ',SETHETA(I,J),')'
            ENDIF
           ENDIF
          ENDIF
 175     CONTINUE
        ENDIF
 170   CONTINUE
      ENDIF
*
*	Other stuff only gets written on first, and possibly
*	final, pass. First, global quantities ...
*
      IF ((CNVRGE.EQ.1).OR.(CNVRGE.EQ.3)) THEN
       IF (NP(8).GT.0) THEN
        WRITE(*,'(/,''Global quantities:'')')
        WRITE(*,'(18(''-''))')
        WRITE(60,'(/,''Global quantities:'')')
        WRITE(60,'(18(''-''))')
        DO 180 I = 1,NP(8)
         GLOBID = GLBCOD(I)/1000
         WRITE(*,'(10X,A30,'':'',1X,F9.4)') GLBTXT(GLOBID),
     +                                              GLBVAL(GLOBID)
         WRITE(60,'(10X,A30,'':'',1X,F9.4)') GLBTXT(GLOBID),
     +                                              GLBVAL(GLOBID)
 180    CONTINUE
       ENDIF
*
*     Now the dispersion parameter
*
       IF ((NU.GT.0.0D0).AND.(CNVRGE.EQ.1)) THEN
        WRITE(*,'(/''Dispersion parameter: '',F8.4,/30(''-''))') NU
        WRITE(60,'(/''Dispersion parameter: '',F8.4,/30(''-''))') NU
       ELSEIF (CNVRGE.EQ.1) THEN
        WRITE(*,'(/''No dispersion parameters defined'')')
        WRITE(60,'(/''No dispersion parameters defined'')')
       ENDIF
*
*	Now stuff about spatial structure. 
*
        IF ((SPMOD.GE.20).OR.(SPMOD.EQ.0)) THEN
         SPDESC = SPATXT(SPMOD)
        ELSE
         SPDESC = SPATXT(SPMOD)
         SPDESC = SPDESC(1:INDEX(SPDESC,'  '))//CORTXT(MDLTYP)
        ENDIF
        WRITE(*,'(/,''Spatial dependence structure:'')')
        WRITE(*,'(30(''-''))')
        WRITE(*,'(''Structure used is '',A50)') SPDESC
        WRITE(60,'(/,''Spatial dependence structure:'')')
        WRITE(60,'(30(''-''))')
        WRITE(60,'(''Structure used is '',A60)') SPDESC
        IF (SPMOD.NE.0) THEN
         DO 190 I=1,NP(10)
          WRITE(*,'(10X,A30,A2,F9.4)') SPPTXT(SPMOD,I),': ',RHO(I)
          WRITE(60,'(10X,A30,A2,F9.4)') SPPTXT(SPMOD,I),': ',RHO(I)
  190    CONTINUE
        ENDIF
       ENDIF
*
*	Check model specification first time round
*
      IF (CNVRGE.EQ.1) THEN
       IF ((PWTFLG.EQ.1).OR.((SPMOD.GT.2).AND.(SPMOD.LT.20))) THEN
        WRITE(*,1) ATTRTXT(1),ATTRTXT(2)
        WRITE(60,1) ATTRTXT(1),ATTRTXT(2)
        WRITE(60,*)
       ENDIF
       IF (NP(8).LT.1) THEN
        WRITE(*,2)
        WRITE(60,2)
        DO 200 I=1,1
         IF (GLBVAL(I).GT.1.0D8) THEN
          WRITE(*,'(10X,A60)') GLBTXT(I)
          WRITE(60,'(10X,A60)') GLBTXT(I)
         ENDIF
 200    CONTINUE
        WRITE(60,*)
       ENDIF
       CALL OKCONT
      ENDIF
      RETURN

 1    FORMAT(/,'**** NOTE: distance-dependent weights (for averages ',
     +'of previous days''',/,'values and / or correlations) will be ',
     +'based on distances calculated from',//,5X,A70,/,'and  ',A70)
 2    FORMAT(//'**** NOTE: the following global quantities have not ',
     +'been defined:',/)
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE SITEDEF(FILNO,FILNM,ATTRTXT,NATTRIB,SITINF,REGCODES,
     +                                SCODES,SITETXT,NSITES,MXN,MXP)
*
*       Reads sites which are required from file FILNM attached to 
*       unit FILNO, defines their 
*       properties (in SITINF), what those properties are (ATTRTXT and 
*       NATTRIB) which subareas they're in (in REGCODES), their names
*       (in SITETXT) and abbreviation codes (in SCODES)
*
      INTEGER LINENO,FILNO,MXN,MXP
      INTEGER NATTRIB,NSITES,REGCODES(MXN)
      DOUBLE PRECISION SITINF(MXN,MXP,0:3)
      CHARACTER ATTRTXT(MXP)*70,SITETXT(MXN)*70,SCODES(MXN)*4
      CHARACTER FILNM*12
****************************************************************************
*       Additional INTEGERS
*       ^^^^^^^^^^^^^^^^^^^
*       NHEAD   - No. of header lines in input file
*	LINENO	- Line number of line we're currently reading
****************************************************************************
      INTEGER NHEAD,I
****************************************************************************
*       Additional CHARACTERS
*       ^^^^^^^^^^^^^^^^^^^^^
*       TXT     - Any text read from file
*       CODE    - 4-character site code read from file
****************************************************************************
      CHARACTER TXT*70,CODE*4
      
      NHEAD = 38
      NSITES = 0

      DO 10 I=1,NHEAD
       READ(FILNO,*)
 10   CONTINUE
*
*       Read attributes
*
      READ(FILNO,*,ERR=96) NATTRIB
      DO 15 I=1,NATTRIB
       READ(FILNO,FMT='(A70)',ERR=97) ATTRTXT(I)
 15   CONTINUE

      READ(FILNO,*)
      LINENO = NHEAD+NATTRIB+2
*
*       Now read site information - we only allow end-of-file 
*       conditions after a complete pair of lines has been read
*
      
 20   LINENO = LINENO + 1
      READ(FILNO,FMT='(A70)',ERR=98,END=90) TXT
      SITETXT(NSITES+1) = TXT
      LINENO = LINENO + 1
      READ(FILNO,FMT=1,ERR=99) CODE,REGCODES(NSITES+1),
     +                  (SITINF(NSITES+1,I,0),I=1,NATTRIB)
*
*       Successful read - increment number of sites by 1 &
*       put into array
*
      NSITES = NSITES + 1
      SCODES(NSITES) = CODE

      GOTO 20

 90   CLOSE(FILNO)
      IF (NSITES.LT.1) CALL READERR(FILNM,0,15)
      IF (NSITES.GT.MXN) CALL READERR(FILNM,0,16)
      RETURN
*
*       Error trapping
*
 96   CALL READERR(FILNM,NHEAD+1,2)
 97   CALL READERR(FILNM,NHEAD+1+I,2)
 98   CALL READERR(FILNM,LINENO,2)
 99   CALL READERR(FILNM,LINENO,2)

 1    FORMAT(A4,I5,50(F10.0))
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE TAUDEF(GLBVAL,GLBCOD,NGLB,THRESH,TRACE,THRTYP,MXP)
*
*     To define a level for `soft', `hard' or `trace' thresholding of 
*     non-negative data. . Arguments:
*
*     GLBVAL    - Array of `global' quantities. The first element 
*                 contains the threshold. DOUBLE PRECISION, input
*     GLBCOD    - Indexing array, linking the quantities in the 
*                 model to the elements of GLBVAL. INTEGER, input
*     NGLB      - Number of global quantities defined in the model.
*                 INTEGER, input.
*     THRESH    - Threshold. DOUBLE PRECISION, output.
*     TRACE     - Threshold, if `trace' thresholding is to be used.
*                 DOUBLE PRECISION, output.
*     THRTYP    - Indicator for type of thresholding. 1 = `soft',
*                 2 = `hard', 0 = `none'. INTEGER, output.
*     MXP       - Used for dimensioning. INTEGER, input
******************************************************************************
      INTEGER MXP
      INTEGER GLBCOD(MXP),NGLB,THRTYP
      DOUBLE PRECISION GLBVAL(MXP),THRESH,TRACE
******************************************************************************
*     Additional INTEGERs
*     ^^^^^^^^^^^^^^^^^^^
*     I         - counter
******************************************************************************
      INTEGER I

*
*     Basic strategy: threshold is zero unless instructed otherwise.
*     If threshold is defined, appropriate element of GLBCOD will
*     be coded as 1000 + x, where x is 1 for `trace' treatment of small
*     values, 2 for `soft' thresholding and 3 for `hard' thresholding.
*     Since `trace' treatment does not involve thresholding, we get
*     THRTYP = x - 1.
*
      THRESH = 0.0D0
      TRACE = 0.0D0
      THRTYP = 0
      DO 100 I=1,NGLB
       IF (GLBCOD(I)/1000.NE.1) GOTO 100
       THRTYP = MOD(GLBCOD(I),1000) - 1
       IF (THRTYP.EQ.0) THEN
        TRACE = GLBVAL(1)
       ELSE
        THRESH = GLBVAL(1)
       ENDIF
       RETURN
 100  CONTINUE

      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE READERR(FILNM,LINE,ERRNO)
*
*       Input error trapping - called if an error of code ERRNO is found
*       on line LINE of file FILNM. Codes for ERRNO are:
*               1       OLD file doesn't exist
*               2       General input error - I've read something I
*                       wasn't expecting
*               3       Input file is in wrong order
*		4	Wrong header length in input file
*              10       No name specified for whole area
*              11       Subareas are not numbered consecutively
*              15       No sites defined
*              16       Too many sites defined
*              17       Requested site attribute not defined
*              18       Insufficient attributes defined for calculation
*                       of inter-site distances.
*              20       Invalid model component
*              21       Illegal interaction 
*	       22	Asked to keep track of too many previous
*			days' rainfall
*              25       Trace threshold required but undefined.
*	       30       Parameter already defined
*	       31	Parameter can't be used with that covariate
*	       32	Required codes haven't been specified
*              33       Invalid nonlinear parameter definition
*              34       Invalid combination of codes
*	       35       Can't estimate a nonlinear parameter with given
*			starting values.
*	       36	Attempt to estimate a parameter that must be 
*                       prespecified.
*	       38	For orthogonal series represenation, upper limit
*			is less than lower limit.
*	       39	Some sites have attributes outside specified
*			limits for orthogonal series representation
*	       40       Too many parameters for available storage
*	       45 	Attempt to define more than 1 spatial structure
*	       46       Invalid spatial structure selected
*	       47       Invalid spatial structure for the response variable
*			you're trying to model (e.g. correlations for
*			logistic regression)
*	       50       Attempt to define more than 1 dispersion parameter
*	       51       No dispersion parameter defined (simulation only)
*	       52       Attempt to define dispersion parameter for 
*                       model that doesn't support it.
*              60       Missing external data (simulation only)
*
      CHARACTER FILNM*12
      INTEGER LINE,ERRNO
 
      IF (ERRNO.EQ.1) THEN
       WRITE(*,1) FILNM
      ELSEIF (ERRNO.EQ.2) THEN
       WRITE(*,2) LINE,FILNM
      ELSEIF (ERRNO.EQ.3) THEN
       WRITE(*,3) LINE,FILNM
      ELSEIF (ERRNO.EQ.4) THEN
       WRITE(*,4) FILNM,LINE
      ELSEIF (ERRNO.EQ.10) THEN
       WRITE(*,10) FILNM
      ELSEIF (ERRNO.EQ.11) THEN
       WRITE(*,11) LINE,FILNM
      ELSEIF (ERRNO.EQ.15) THEN
       WRITE(*,15) FILNM
      ELSEIF (ERRNO.EQ.16) THEN
       WRITE(*,16) FILNM
      ELSEIF (ERRNO.EQ.17) THEN
       WRITE(*,17) LINE,FILNM
      ELSEIF (ERRNO.EQ.18) THEN
       WRITE(*,18) LINE,FILNM
      ELSEIF (ERRNO.EQ.20) THEN
       WRITE(*,20) LINE,FILNM
      ELSEIF (ERRNO.EQ.21) THEN
       WRITE(*,21) LINE,FILNM
      ELSEIF (ERRNO.EQ.22) THEN
       WRITE(*,22) LINE,FILNM
      ELSEIF (ERRNO.EQ.25) THEN
       WRITE(*,25) FILNM
      ELSEIF (ERRNO.EQ.30) THEN
       WRITE(*,30) LINE,FILNM
      ELSEIF (ERRNO.EQ.31) THEN
       WRITE(*,31) LINE,FILNM
      ELSEIF (ERRNO.EQ.32) THEN
       WRITE(*,32) LINE,FILNM
      ELSEIF (ERRNO.EQ.33) THEN
       WRITE(*,33) LINE,FILNM
      ELSEIF (ERRNO.EQ.34) THEN
       WRITE(*,34) LINE,FILNM
      ELSEIF (ERRNO.EQ.35) THEN
       WRITE(*,35) LINE,FILNM
      ELSEIF (ERRNO.EQ.36) THEN
       WRITE(*,36) LINE,FILNM
      ELSEIF (ERRNO.EQ.38) THEN
       WRITE(*,38) LINE,FILNM
      ELSEIF (ERRNO.EQ.39) THEN
       WRITE(*,39) LINE,FILNM
      ELSEIF (ERRNO.EQ.40) THEN
       WRITE(*,40) FILNM,LINE
      ELSEIF (ERRNO.EQ.45) THEN
       WRITE(*,45) LINE,FILNM
      ELSEIF (ERRNO.EQ.46) THEN
       WRITE(*,46) LINE
      ELSEIF (ERRNO.EQ.47) THEN
       WRITE(*,47) LINE,FILNM
      ELSEIF (ERRNO.EQ.50) THEN
       WRITE(*,50) LINE,FILNM
      ELSEIF (ERRNO.EQ.51) THEN
       WRITE(*,51) FILNM
      ELSEIF (ERRNO.EQ.52) THEN
       WRITE(*,52) LINE,FILNM
      ELSEIF (ERRNO.EQ.60) THEN
       WRITE(*,60) LINE
      ELSE
       WRITE(*,99) FILNM
      ENDIF
      STOP

 1    FORMAT(/'****ERROR**** Input file ',A12,' does not exist') 
 2    FORMAT(/'****ERROR**** Input error while reading line ',
     +I3,' of file ',A12,'.',/,'This *may* (but no promises!) ',
     +'be because you have at least one empty',/,'line at the end ',
     +'of the file. If this is the case, delete the ',/,
     +'offending line(s) and try again.')
 3    FORMAT(/'****ERROR**** Line ',I2,' of file ',A12,
     +' is out of order')
 4    FORMAT(/'****ERROR**** There is a problem with input file ',
     +A12,' - my guess is',
     +/,'the number of header rows is wrong. There should be ',I2,
     +' - see the examples',
     +/,'in the program distribution for a correct specification.')
 10   FORMAT(/'****ERROR**** Region 0 (entire area) is not defined',   
     +' in file ',A12)
 11   FORMAT(/'****ERROR**** Subareas are not consecutive at line ',
     +I2,' of file ',A12)
 15   FORMAT(/'****ERROR**** No sites defined in file ',A12)
 16   FORMAT(/'****ERROR**** No. of sites defined in file ',A12,
     +' exceeds available storage.',/,'Contact REC please.')
 17   FORMAT(/'****ERROR**** The site attribute requested on line ',
     +I3,' of file ',A12,/,' has not been defined.')
 18   FORMAT(/'****ERROR**** On line ',I3,' of file ',A12,/,
     +' you ask for distance-weighted',/,'averages of previous ',
     +'days'' values. I need both X and Y site co-ordinates to',/,
     +'be defined in file siteinfo.def.')
 20   FORMAT(/'****ERROR**** Invalid model component on line ',
     +I3,' of file ',A12)
 21   FORMAT(/'****ERROR**** Illegal interaction specified on line ',
     +I3,' of file ',A12)
 22   FORMAT(/'****ERROR**** On line ',I3,' of file ',A12,', you ',
     +'have asked me',/,
     +'to keep track of too many previous days'' values. ',
     +'I can''t',/,'remember more than 10!')
 25   FORMAT(/'****ERROR**** In file ',A12,', some covariates are ',
     +'`trace''',/,'indicators, but you haven''t defined a ',
     +'`trace'' threshold',/,'in the `global quantities'' section',
     +' of the file.')
 30   FORMAT(/'****ERROR**** Parameter on line ',I3,' of file ',A12,
     +' is already defined.')
 31   FORMAT(/'****ERROR**** On line ',I3,' of file ',A12,
     +', parameters',
     +/'are defined for a function which doesn''t exist!')
 32   FORMAT(/'****ERROR**** On line ',I3,' of file ',A12,
     +', you haven''t',
     +/'specified enough information to identify what you want.')
 33   FORMAT(/'****ERROR**** The nonlinear parameter definition ',
     +'on line ',I3,' of file ',/,A12,' is incorrect. It refers ',
     +'either to a covariate that isn''t',/,
     +'a main effect, or to a parameter that doesn''t exist.')
 34   FORMAT(/'****ERROR**** The combination of codes on line ',I3,
     +' of file',/,A12,' is invalid.')
 35   FORMAT(/'****ERROR**** On line ',I3,' of file ',A12,
     +', you ask',
     +/'for estimation of a nonlinear parameter, but you''ve given ',
     +'zero ',/,'as the coefficient of the corresponding covariate!')
 36   FORMAT(/'****ERROR**** On line ',I3,' of file ',A12,
     +', you''ve',
     +/'asked me to estimate a parameter that should be ',
     +'regarded as known.')
 38   FORMAT('****ERROR**** You have asked for orthogonal series ',
     +'representation',
     +/'of a function over an interval whose upper limit is not ',
     +'greater than',
     +/'its lower limit (line ',I3,' of file ',A12,')')
 39   FORMAT('****ERROR**** Site number ',I3,' has an attribute ',
     +'outside the range',
     +/'you have specified for orthogonal series representation',
     +' in file ',A12,'.')
 40   FORMAT(/'****ERROR**** Model specified in file ',A12,
     +' contains ',/,
     +'more parameters than I have available storage (current ',
     +'maximum is ',I3,')',/,
     +'Increase the storage and recompile.')
 45   FORMAT(/'****ERROR**** On line ',I3,' of file ',A12,
     +', you specify',
     +/'a spatial structure which is different from the one you ',
     +'specified',
     +/'previously in the same file!')
 46   FORMAT(/'****ERROR**** you''ve asked for spatial correlation ',
     +'model ',I2,' - ',
     +/'I don''t have one of those!')
 47   FORMAT(/'****ERROR**** on line ',I3,' of file ',A12,
     +', you specify',
     +/'a spatial structure which is inconsistent with the response',
     +' variable',
     +/'you''re trying to model.')
 50   FORMAT(/'****ERROR**** On line ',I3,' of file ',A12,
     +' you''ve specified a',
     +/'second dispersion parameter - I can only deal with one.')
 51   FORMAT(/'****ERROR**** In file ',A12,', you haven''t defined ',
     +'a dispersion parameter.')
 52   FORMAT(/'****ERROR**** On line ',I3,' of file ',A12,
     +' you''ve specified a',/,'dispersion parameter for a model that ',
     +'doesn''t have one.')
 60   FORMAT(/'****ERROR**** Missing external predictor data (current ',
     +'date is ',I8,')')
 99   FORMAT(/'****ERROR**** Unidentified input error in file ',A12)

      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE OKCONT
*
*       Check whether it's OK to continue
*
      CHARACTER CONT*1
      WRITE(*,'(//,''OK to continue (Y/N, default Y)?'')')
      READ(*,'(A1)') CONT
      IF ( (CONT.EQ.'N').OR.(CONT.EQ.'n') ) STOP
      END
******************************************************************************
******************************************************************************
******************************************************************************
      INTEGER FUNCTION DAYS_IN_MONTH(MONTH,YEAR)
      IMPLICIT NONE
C
C Author       : Kevin Black
C Date written : January 1985
C Abstract     :
C
C Integer function returns the number of days in the month MONTH for the
C year YEAR (leap years are accounted for)
C
      INTEGER MONTH,YEAR
C
C Local variables
C
      INTEGER NDAYS(12)
      DATA NDAYS/31,28,31,30,31,30,31,31,30,31,30,31/
C
C Get number of days for months other than February
C
      IF(MONTH.NE.2)THEN
	 DAYS_IN_MONTH=NDAYS(MONTH)
C
C To be a leap year the year number must be divisble by 4
C but not by 100. However if the year is divisble by 4
C and by 400 then it is a leap year (this being the exception
C to the 100 divisibility rule).
C
      ELSE IF(YEAR/4*4.EQ.YEAR.AND.
     *        (YEAR/400*400.EQ.YEAR.OR.
     *         YEAR/100*100.NE.YEAR))THEN
	      DAYS_IN_MONTH=29
	   ELSE
	      DAYS_IN_MONTH=28
	   ENDIF
      RETURN
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE COVSET(COVS,NP,NSITES,COVCODE,TWO,THREE,
     + GLBCOD,SITINF,YEAR,MONTH,DAY,PREVIOUS,PWTIDX,MXLAG,GLBVAL,
     + BETA,THETA,TRACE,RECALC,FORCE,MISSFL,ICHECK,MXN,MXP,SIM,
     + MISSST)
*******************************************************************************
*       Calculates covariates in model for 1 days' data.
*******************************************************************************
*       INTEGER variables
*       ^^^^^^^^^^^^^^^^^
*       NP      - No. of parameters in the model. Input
*       NSITES  - No. of sites. Input
*       COVCODE - Codes for covariates in the model. Input
*       TWO  - list of 2-way interactions. Input
*       THREE- list of 3-way interactions. Input
*       GLBCOD  - Defines `global' quantities in GLBVAL. Input
*	MXLAG	- We'll chuck out any observations for which there are
*		  missing values during any of the MXLAG immediately 
*		  preceding days.
*	YEAR  	- current year. Input
*	MONTH   - current month. Input
*       DAY	- current day. Input
*       PWTIDX  - Index for locating parameters relating to nonlinear 
*                 weighting schemes for previous days' values.
*       WTSCHM  - Weighting scheme required.
*	OLDYR	- Value of YEAR last time through (do we have to recalculate?)
*	OLDMN	- Value of MONTH last time through (ditto)
*	RECALC	- Indicates whether we're calculating everything (0) or
*		  just things associated with nonlinear functions (>0)
*	FORCE	- Forces recalculation of everything even if the year and
*		  month are the same as on previous call (for initialising
*		  COVS in programs where more than 1 model is involved)
*	MISSFL	- Array of flags indicating whether there's any
*		  missing data at any of the sites today. Output.
*       MXN     - Maximum no. of sites. Input
*       MXP     - Maximum no. of parameters. Input
*       SIM     - 1 if we're simulating, 0 otherwise (used to halt
*                 simulation if data on external predictors is missing).
*       MISSST  - Indicator - should we set MISSFL in this routine?
*	IDX1	- Covariate no. for a nonlinear parameter we're 
*		  currently dealing with
*	IDX2	- parameter no, relative to the covariate
*       EXTLAG  - lag, for external predictors
*       YMISS } - indicators telling us whether any required external
*       MMISS }   predictors are missing
*       DMISS
*       ICHECK  - Indicator to tell us whether parametrisation has been 
*                 checked (only need to carry out checks on first call
*                 to this routine)
*******************************************************************************
      INTEGER WTSCHM,MXN,MXP,SIM,MISSST
      INTEGER NP(10),NSITES,COVCODE(MXP),TWO(MXP,2),THREE(MXP,3)
      INTEGER GLBCOD(MXP),YEAR,MONTH,DAY,OLDYR,OLDMN
      INTEGER RECALC,FORCE,MISSFL(MXN),MXLAG,PWTIDX(MXP,0:3)
      INTEGER I,J,K,IDX1,IDX2,EXTLAG,YMISS,MMISS,DMISS,ICHECK
*******************************************************************************
*       DOUBLE PRECISION variables
*       ^^^^^^^^^^^^^^^^^^^^^^^^^^
*       COVS    - Values of covariates at each site. Input/output. The routine
*		  modifies only those elements which need to be modified NB THIS
*		  REQUIRES THAT ON ENTRY, THE ELEMENTS OF COVS 
*		  CORRESPONDING TO YEAR AND MONTHLY EFFECTS HAVE NOT BEEN
*		  UPDATED SINCE THE PREVIOUS CALL, IF WE'RE IN THE SAME YEAR/
*		  MONTH AS WE WERE FOR THE PREVIOUS CALL!
*       SITINF - Raw information regarding each site
*	PREVIOUS - previous days' rainfall values
*       GLBVAL  - Values of `global' quantities
*	TRACE	- trace threshold
*	BETA	- Parameter vector
*	THETA	- array containing parameters in nonlinear transformations,
*		  arranged to line up with their covariates. Values in
*		  positions NP(6)+1 to NP(7) actually contain derivatives of 
*		  nonlinear functions wrt the parameters.
*	TRENDFN	- Value of long-term trend, + its derivatives wrt any
*		  nonlinear parameters we're trying to estimate. Element 0
*		  is the value, 1 and 2 are derivatives.
*	MONFN	- FUNCTION used for monthly contributions
*	DYPRED	- Predictor (& possible derivatives wrt nonlinear parameters)
*                 corresponding to previous days.
*       EXTDY   - value of external forcing variable for today
*	TMP	- temporary storage
*******************************************************************************
      DOUBLE PRECISION COVS(MXN,MXP),SITINF(MXN,MXP,0:3)
      DOUBLE PRECISION PREVIOUS(MXN,10)
      DOUBLE PRECISION TRENDFN(0:2),MONFN,DYPRED(0:3),EXTDY
      DOUBLE PRECISION GLBVAL(MXP),TRACE
      DOUBLE PRECISION BETA(0:MXP),THETA(MXP,3)
      DOUBLE PRECISION TMP
*
*	Remeber year and month effects in case we want to use them again
*	(and start them all off as zeroes). Also remember whether we've
*       checked the model.
*
      SAVE OLDYR,OLDMN,YMISS,MMISS,DMISS
      DATA OLDYR,OLDMN,YMISS,MMISS,DMISS /5*0/
*
*	If we're checking for missing data, initialise MISSFL
*       (assume no missing data) ...
*
      IF (MISSST.EQ.1) THEN
       DO 40 I=1,NSITES
        MISSFL(I) = 0
 40    CONTINUE
      ENDIF
*
*	This routine does all sites for 1 day; therefore we only need 
*       to compute year and month effects, and `external' daily effects, 
*       once. (save on flops). Don't do any parameters that we
*	don't have to! Values are only set if the year/month changes.
*       This means that these elements of COVS must NOT be changed 
*       between calls. First years (trends and `external forcings'
*       are dealt with differently. NB the need to deal with missing
*       data when external forcings are involved - effectively, we
*       just go onto the next year).
*
      DO 50 J=NP(1)+1,NP(2)
       IF ((RECALC.GT.0).AND.(THETA(J,1).GT.1.0D8)) GOTO 50
       IF ((YEAR.NE.OLDYR).OR.(FORCE.EQ.1)) THEN
        IF (MOD(IABS(COVCODE(J)),1000).LE.50) THEN
         CALL TRENDSB(TRENDFN,YEAR,J,COVCODE(J),THETA,ICHECK,MXP)
        ELSE
         EXTLAG = COVCODE(J)/1000
         CALL EXTSET(31,1,MOD(IABS(COVCODE(J)),1000)-50,
     +                    DAY,MONTH,YEAR,EXTLAG,TRENDFN(0),YMISS)
        ENDIF
        DO 55 I=1,NSITES
         COVS(I,J) = TRENDFN(0)
 55     CONTINUE
*
*	There may be nonlinear functions floating round here,
*	so we'll take the opportunity to set the derivatives for their
*	parameters. NB we know whether or not a nonlinear parameter is
*	to do with the current predictor because the predictor is
*	given by COVCODE/1000 (integer division). NB ALSO: for the moment,
*	we put the value of the derivative in the row of THETA
*	corresponding to the nonlinear parameter (this does not conflict
*	with anything else, since actual parameter values of THETA are
*	stored in the row of the covariate to which they correspond).
*	This is necessary to keep track of what the derivatives actually 
*	are when interaction terms are involved, without having to 
*	recalculate every time.
*
        DO 57 K=NP(6)+1,NP(7)
         IF (COVCODE(K)/1000.EQ.J) THEN
          IDX2 = MOD(COVCODE(K),1000)
          THETA(K,IDX2) = TRENDFN(IDX2)
         ENDIF
 57     CONTINUE
       ENDIF
 50   CONTINUE
      OLDYR = YEAR
*
*       Monthly, then daily, effects. Skip if we're only RECALCulating 
*       nonlinear contributions. Again, `external' monthly contributions
*       are dealt with differently.
*
      IF (RECALC.GT.0) GOTO 99
      DO 60 J=NP(2)+1,NP(3)
       IF ((MONTH.NE.OLDMN).OR.(FORCE.EQ.1)) THEN
        IF (MOD(IABS(COVCODE(J)),1000).LE.50) THEN
         CALL MONSB(MONFN,MONTH,YEAR,COVCODE(J))
        ELSE
         EXTLAG = COVCODE(J)/1000
         CALL EXTSET(32,2,MOD(IABS(COVCODE(J)),1000)-50,
     +                     DAY,MONTH,YEAR,EXTLAG,MONFN,MMISS)
        ENDIF
        DO 65 I=1,NSITES
         COVS(I,J) = MONFN
 65     CONTINUE
       ENDIF
 60   CONTINUE

      DO 70 J=NP(3)+1,NP(4)
       IF (.NOT.(MOD(IABS(COVCODE(J)),1000).LE.10)) THEN
*
*     Have to pass PREVIOUS array over here, even though we're not going
*     to use it.
*
        IF (MOD(IABS(COVCODE(J)),1000).LE.50) THEN
         CALL DYSET(DAY,MONTH,YEAR,PREVIOUS,0,NSITES,TRACE,
     +     COVCODE(J),THETA,PWTIDX,SITINF,ICHECK,DYPRED,MXN,MXP)
         EXTDY = DYPRED(0)
        ELSE
         EXTLAG = COVCODE(J)/1000
         CALL EXTSET(33,3,MOD(IABS(COVCODE(J)),1000)-50,
     +                     DAY,MONTH,YEAR,EXTLAG,EXTDY,DMISS)
        ENDIF
        DO 75 I=1,NSITES
         COVS(I,J) = EXTDY
 75     CONTINUE
       ENDIF
 70   CONTINUE

 99   OLDMN = MONTH
*
*     Missing external predictors? If so, no point continuing - set
*     everything to missing and return.
*
      IF( (YMISS.EQ.1).OR.(MMISS.EQ.1).OR.(DMISS.EQ.1)) THEN 
       IF (MISSST.EQ.1) THEN
        DO 80 I=1,NSITES
         MISSFL(I) = 1
 80     CONTINUE
       ENDIF
       IF (SIM.EQ.1) THEN
        CALL READERR('            ',DAY+(100*MONTH)+(10000*YEAR),60)
       ELSE
        RETURN
       ENDIF
      ENDIF
*
*       That's it for effects that just have to be computed once. Now 
*       the remainder. Start with main effects. If we're recomputing
*       at a site which has missing predictor data according to the
*       MISSFL array, skip the whole section.
*
      DO 100 I=1,NSITES
       IF ((MISSST.EQ.0).AND.(MISSFL(I).EQ.1)) GOTO 100
*
*	Site effects. May be some nonlinear parameters to estimate here, so
*	as before we copy the derivatives into the appropriate place in
*	THETA (NB we're now doing a site at a time, so these elements of
*	THETA will get changed around, but that doesn't matter since they're
*	not used anywhere else).
*
       DO 110 J=1,NP(1)
        IF ((RECALC.GT.0).AND.(THETA(J,1).GT.1.0D8)) GOTO 110
	COVS(I,J) = SITINF(I,COVCODE(J),0)
        DO 111 K=NP(6)+1,NP(7)
         IF (COVCODE(K)/1000.EQ.J) THEN
          IDX2 = MOD(COVCODE(K),1000)
          THETA(K,IDX2) = SITINF(I,COVCODE(J),IDX2)
         ENDIF
 111     CONTINUE
 110   CONTINUE
*
*       Daily effects. At this point we're just setting previous days' 
*       values - `external' daily effects don't vary between sites and 
*       have already been set. If any previous values are missing, set 
*       a MISSING flag for this site and go on to the next. The effect 
*       will be not to alter anything that was previously in COVS, which 
*       should therefore be initialised in the calling program if used 
*       for simulation. 
*
 119   DO 120 K=1,MXLAG
        IF (PREVIOUS(I,K).LT.0.0D0) THEN
         IF (MISSST.EQ.1) MISSFL(I) = 1
         GOTO 100
        ENDIF
 120   CONTINUE

       DO 130 J=NP(3)+1,NP(4)
        IF (MOD(IABS(COVCODE(J)),1000).LE.10) THEN
*
*     There may be nonlinear parameters to estimate here, associated 
*     (for example) with weighted averages of previous days' values.
*     We can tell whether or not this is the case, because there will
*     be non-zero pointers in PWTIDX. If so, we copy their derivatives 
*     into a spare row of THETA (this will get overwritten as we move 
*     through the sites, but that's OK as we're basically using it for 
*     temporary storage, a site at a time). There's an additional 
*     complication here, which is that we will generally have a single
*     parametrisation that covers various different lagged values. This
*     will give rise to several different sets of derivatives, so we need
*     more than one spare row of THETA. Creativity required: if the
*     current predictor is the one that the weighting scheme is nominally
*     attached to, use a spare row in the `nonlinear' part of THETA. 
*     Otherwise, use the row corresponding to the nonlinear part (it
*     doesn't get used for anything else).
*
         WTSCHM = MOD(COVCODE(J)/10000,10)
         IF (RECALC.GT.0) THEN
*
*     Need to break this down into 2 bits, to avoid referencing
*     PWTIDX(0,*) which isn't defined
*
          IF (WTSCHM.EQ.0) GOTO 130
          IF ((PWTIDX(WTSCHM,1).EQ.0).AND.
     +       (PWTIDX(WTSCHM,2).EQ.0).AND.
     +       (PWTIDX(WTSCHM,3).EQ.0)) GOTO 130
         ENDIF
         CALL DYSET(DAY,MONTH,YEAR,PREVIOUS,I,NSITES,TRACE,
     +       COVCODE(J),THETA,PWTIDX,SITINF,ICHECK,DYPRED,MXN,MXP)
         COVS(I,J) = DYPRED(0)
         IF (WTSCHM.EQ.0) GOTO 130
         DO 135 K=1,3
          IF (PWTIDX(WTSCHM,K).GT.0) THEN
           IF (J.EQ.PWTIDX(WTSCHM,0)) THEN
            THETA(PWTIDX(WTSCHM,K),K) = DYPRED(K)
           ELSE
            THETA(J,K) = DYPRED(K)
           ENDIF
          ENDIF
 135     CONTINUE
        ENDIF
 130   CONTINUE
*
*       Now 2-way interactions. Just do the ones for which main
*	effects have been defined (not all when we're just recalculating).
*	If we're recaclulating, we only do it when there's a nonlinear 
*	parameter in an associated main effect.
*
 139   DO 140 J=NP(4)+1,NP(5)
        IF ((RECALC.GT.0).AND.
     +      (THETA(TWO(J,1),1).GT.1.0D8).AND.
     +      (THETA(TWO(J,2),1).GT.1.0D8) ) GOTO 140
        COVS(I,J) = COVS(I,TWO(J,1))*COVS(I,TWO(J,2))
 140   CONTINUE  
*
*       And 3-way interactions
*
       DO 150 J=NP(5)+1,NP(6)
        IF ((RECALC.GT.0).AND.
     +      (THETA(THREE(J,1),1).GT.1.0D8).AND.
     +      (THETA(THREE(J,2),1).GT.1.0D8).AND.
     +      (THETA(THREE(J,3),1).GT.1.0D8) ) GOTO 150
        COVS(I,J) = COVS(I,THREE(J,1))
     +              *COVS(I,THREE(J,2))
     +              *COVS(I,THREE(J,3))
 150   CONTINUE  
*
*	Now the 'dummy' elements corresponding to derivatives of 
*	nonlinear parameters. We need to take the derivatives of the
*	nonlinear function itself (set into spare bits of THETA earlier),
*	and scale as appropriate: they appear in the linear predictor as 
*	multiples of other parameters and (where interactions are 
*	involved) covariates.
*	
       DO 160 J=NP(6)+1,NP(7)
        IDX1 = COVCODE(J)/1000
        IDX2 = MOD(COVCODE(J),1000)
        CALL NLCNTR(IDX1,NP,BETA,TWO,THREE,COVS,I,
     +                                             MXN,MXP,TMP)
        COVS(I,J) = THETA(J,IDX2)*TMP
*
*       Parameters in weighting schemes are fiddly, because they may 
*       relate to several main effects rather than just one. So 
*       now we have to go through all the *other* previous days' 
*       predictors looking for contributions to this parameter ...
*
        WTSCHM = 0
        IF ((IDX1.GT.NP(3)).AND.(IDX1.LE.NP(4))) THEN
         IF (MOD(IABS(COVCODE(J)),1000).LE.10)
     +                    WTSCHM = MOD(COVCODE(IDX1)/10000,10)
        ENDIF
        IF (WTSCHM.NE.0) THEN
         DO 165 IDX1 = NP(3)+1,NP(4)
          IF (MOD(COVCODE(IDX1)/10000,10).NE.WTSCHM) GOTO 165
          IF (MOD(IABS(COVCODE(IDX1)),1000).GT.10) GOTO 165
          IF (IDX1.EQ.COVCODE(J)/1000) GOTO 165
          CALL NLCNTR(IDX1,NP,BETA,TWO,THREE,COVS,I,
     +                                             MXN,MXP,TMP)
*
*     For these contributions, the derivatives of THETA are stored
*     in the rows corresponding to main effects
*
          COVS(I,J) = COVS(I,J) + (THETA(IDX1,IDX2)*TMP)
 165     CONTINUE
        ENDIF
 160   CONTINUE
 100  CONTINUE

      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE NLCNTR(PREDNO,NP,BETA,TWO,THREE,COVS,SITE,
     +                                                 MXN,MXP,XOUT)
*******************************************************************************
*     To calculate contributions to a `dummy' predictor representing 
*     derivative wrt a nonlinear parameter, that are associated with 
*     a given main effect in a GLM. Arguments:
*
*     PREDNO -       Number of main effect required. INTEGER, input
*     NP     -       Index of model predictors in different categories.
*                    INTEGER, input.
*     BETA   -       Curremt estimate of coefficient vector. DOUBLE, input.
*     TWO -       Table of 2-way interactions. INTEGER, input.
*     THREE -     Guess.
*     COVS   -       Array of covariate values. DOUBLE, input.
*     SITE   -       Current site.
*     MXN,MXP -      Dimensioning. INTEGER, input.
*     XOUT   -       Required contribution. DOUBLE, output.
*******************************************************************************
      INTEGER PREDNO,NP(10),MXN,MXP,SITE
      INTEGER TWO(MXP,2),THREE(MXP,3)
      DOUBLE PRECISION BETA(0:MXP),COVS(MXN,MXP),XOUT
*******************************************************************************
*     Additional INTEGERs
*     ^^^^^^^^^^^^^^^^^^^
*     I    - counter
*******************************************************************************
      INTEGER I

*
*     Check that PREDNO is a main effect
*
      IF (PREDNO.GT.NP(4)) GOTO 990

      XOUT = BETA(PREDNO)
      DO 100 I=NP(4)+1,NP(5)
       IF ( (TWO(I,1).EQ.PREDNO) ) THEN
        XOUT = XOUT + (BETA(I) * COVS(SITE,TWO(I,2)))
       ELSEIF ( (TWO(I,2).EQ.PREDNO) ) THEN
        XOUT = XOUT + (BETA(I) * COVS(SITE,TWO(I,1)))
       ENDIF
 100  CONTINUE

      DO 120 I=NP(5)+1,NP(6)
       IF ( (THREE(I,1).EQ.PREDNO) ) THEN
        XOUT = XOUT + 
     +      (BETA(I) * COVS(SITE,THREE(I,2)) * 
     +                 COVS(SITE,THREE(I,3)))
       ELSEIF ( (THREE(I,2).EQ.PREDNO) ) THEN
        XOUT = XOUT + 
     +      (BETA(I) * COVS(SITE,THREE(I,1)) * 
     +                 COVS(SITE,THREE(I,3)))
       ELSEIF ( (THREE(I,3).EQ.PREDNO) ) THEN
        XOUT = XOUT + 
     +      (BETA(I) * COVS(SITE,THREE(I,1)) *
     +                 COVS(SITE,THREE(I,2)))
       ENDIF
 120  CONTINUE

      RETURN

*
*     Error trapping
*
 990  WRITE(*,1)
      STOP
 1    FORMAT('/****ERROR**** in routine NLCNTR, PREDNO is not ',
     +'a main effect.',/,'This is a programming error - contact ',
     +'REC.')
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE SCROPN(ACCESS,FORM,RECL,FILNO)
*******************************************************************************
*       Finds an available file handle and opens a SCRATCH file
*	connected to that handle.
*******************************************************************************
*	INTEGER variables
*	^^^^^^^^^^^^^^^^^
*	RECL	Record length - mandatory if ACCESS = 'DIRECT', ignored
*		otherwise. Input.
*	FILNO	Number of file when it is opened. User inputs an initial
*		value, then it is incremented until a vacant file handle 
*		is found.
*******************************************************************************
      INTEGER RECL,FILNO
*******************************************************************************
*	CHARACTER variables
*	^^^^^^^^^^^^^^^^^^^
*	ACCESS	Access method - either 'DIRECT' or 'SEQUENTIAL'
*	FORM	Format status - either 'FORMATTED' or 'UNFORMATTED'
*******************************************************************************
      CHARACTER ACCESS*10,FORM*11
*******************************************************************************
*	LOGICAL variables
*	^^^^^^^^^^^^^^^^^
*	ISOPEN	TRUE if channel FILNO is already open, FALSE otherwise
*******************************************************************************
      LOGICAL ISOPEN
*******************************************************************************

 10   INQUIRE(UNIT=FILNO,OPENED=ISOPEN)
      IF (ISOPEN) THEN
       FILNO = FILNO + 1
       GOTO 10
      ENDIF

*
*       Most compilers ignore the RECL argument if ACCESS=SEQUENTIAL,
*       but I've had problems with gfortran - hence slightly 
*       clunky IF structure
*
      IF (ACCESS.EQ.'DIRECT    ') THEN
       OPEN(FILNO,STATUS='SCRATCH',ACCESS=ACCESS,FORM=FORM,RECL=RECL)
      ELSE
       OPEN(FILNO,STATUS='SCRATCH',ACCESS=ACCESS,FORM=FORM)
      ENDIF

      END
***************************************************************************
***************************************************************************
***************************************************************************
      SUBROUTINE BCALC(P,ALPHA,A,B)
***************************************************************************
*	To find the value of B in the conditionally independent 2 weather
*	state model for rainfall occurrence (called from routine WDALLOC),
*	such that the overall probability of a site being wet is P (input).
*	ALPHA is the probability of a wet day (input). log(A) is the
*	coefficient of x (0/1 weather state) in the model (input). B is 
*	output.
*	There is a unique value of B which makes the marginal probability
*	correct. It is obtained by taking one of the roots (we know
*	which one) of a quadratic. We use the numerically robust algorithm
*	suggested by Press et al, Numerical Recipes, Section 5.6.
*
*	This routine is called by fit_logi.f and simrain.f
***************************************************************************
      DOUBLE PRECISION P,ALPHA,A,B
***************************************************************************
*	Extra DOUBLEs
*	^^^^^^^^^^^^^
*	TMP	- Temporary storage
*	QUADA }	- Coefficients in quadratic equation to determine B
*	QUADB }	
*	QUADC }
***************************************************************************
      DOUBLE PRECISION QUADA,QUADB,QUADC,TMP
***************************************************************************
      QUADA = A*P
      QUADB = 1.0D0 + (ALPHA*(A-1.0D0)) - (P*(A+1.0D0))
      QUADC = P-1.0D0
      TMP = DSQRT( (QUADB**2)-(4.0D0*QUADA*QUADC) )
      TMP = -( QUADB + DSIGN(TMP,QUADB) )/ 2.0D0
      IF (QUADB.LT.0.0D0) THEN
       B = TMP/QUADA
      ELSE
       B = QUADC/TMP
      ENDIF
      END
***************************************************************************
***************************************************************************
***************************************************************************
      SUBROUTINE ANSPRM(NU,MEAN,VAR)
*
*       Finds mean and variance of a normal distribution such that when
*       cubed, the result will behave like a Gamma random variable with
*	a mean of 1 and a variance of 1/NU. This basically involves 
*	(i) working with a linear transformation of the underlying normal 
*	and finding the mean and variance of that - there is an explicit 
*	formula for the square of the mean at each value of the variance 
*	so we then have to just solve numerically for the variance (ii) 
*	backtransforming to get the numbers we want. 

*
*       DOUBLE PRECISION variables
*       ^^^^^^^^^^^^^^^^^^^^^^^^^^
*       NU      - shape parameter of gamma distribution
*       MEAN    - required mean of Anscombe residuals
*       VAR     - required variance
*       LLIM }  - lower and upper limits used when solving for MEAN and
*       ULIM }    VAR
*       TOL     - convergence criterion when solving for MEAN and VAR
*       F1,F2   - values of function used to solve for MEAN and VAR
*       MID     - new estimate of VAR (using bisection - tried regula falsi
*                 but it was hugely unstable)
*       FMID    - function value at MID
*
      DOUBLE PRECISION NU,MEAN,VAR
      DOUBLE PRECISION LLIM,ULIM,TOL,F1,F2,MID,FMID

      TOL = 1.0D-8
*
*       For a solution to exist there is an upper bound on VAR. 
*
      LLIM = 1.0D-8
      ULIM = (1.0D0/(15.0D0*NU))**(1.0D0/3.0D0)
*
*       Evaluate the function at the boundaries
*
      VAR = LLIM
      CALL ANSFN(F1,MEAN,LLIM,NU)
      CALL ANSFN(F2,MEAN,ULIM,NU)
*
*       Check that the root is bracketed ...
*
      IF (F1*F2.GT.0.0D0) THEN
       WRITE(*,1)
       STOP
      ENDIF
*
*       Compute midpoint (unsophisticated but stable, which regula 
*       falsi WASN'T)
*
 20   MID = ( LLIM+ULIM )/2.0D0
      CALL ANSFN(FMID,MEAN,MID,NU)
      IF (FMID*F1.LT.0.0D0) THEN
       ULIM = MID
       F2 = FMID
      ELSEIF (FMID*F2.LT.0.0D0) THEN
       LLIM = MID
       F1 = FMID
      ENDIF
      VAR = MID
      IF ( ((ULIM-LLIM).GT.TOL).OR.(DABS(F2-F1).GT.TOL) ) GOTO 20

      RETURN

 1    FORMAT('****ERROR**** Can''t solve for the mean and variance',
     +' of Anscombe residuals.',/,'Please contact REC.')
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE ANSFN(F,MEAN,VAR,NU)
*
*       Evaluates the function F, which we're trying to set to zero
*       when calculating the mean and variance of Anscombe residuals
*       for a given value of NU. F is strictly a function of VAR only;
*       we also calculate the corresponding value of MEAN in this
*       subroutine. A,B and C are coefficients in a quadratic equation
*       for MU squared, which always has positive roots providing 
*       VAR is positive. 
*
      DOUBLE PRECISION F,MEAN,VAR,NU,A,B,C
      
*
*       First calculate MEAN. Context (+probably maths later on, 
*       if I thought about it) dictates MEAN > 0
*
      A = 9.0D0*VAR
      B = 36.0D0*(VAR**2)
      C = (15.0D0*(VAR**3)) - (1.0D0/NU)
*
*       Compute square of mean by solving quadratic
*
      MEAN = (-B+DSQRT( (B**2)-(4.0D0*A*C) ) )/(2.0D0*A)
*
*       If < 0, it's probably a rounding error and should be zero
*
      IF (MEAN.LT.0.0D0) THEN
       IF (DABS(MEAN).LT.1.0D-10) THEN
	MEAN = 0.0D0
       ELSE
	WRITE(*,1)
	STOP
       ENDIF
*
*       Otherwise we just need to square root whatever we've got
*
      ELSE
       MEAN = DSQRT(MEAN)
      ENDIF
*
*     Now calculate the corresponding function value
*
      F = MEAN*( (3.0D0*VAR) + (MEAN**2) ) - 1.0D0

 1    FORMAT('****ERROR**** I''ve just tried to square root a ',
     +'negative number in routine ANSFN.',/,
     +'It''s too big to be a rounding error - please contact REC.')
      END
***************************************************************************
***************************************************************************
***************************************************************************
      SUBROUTINE PRGERR(ROUTIN,SRCFIL,ERRNO)
*
*	To spit out an error message when programming errors are
*	detected. Arguments (all input):
*	ROUTIN	- Name of routine where error was detected. CHARACTER*8
*	SRCFIL	- Name of source file containing that routine. CHARACTER*12
*	ERRNO	- Identifies error. Possible values:
*			1 - Incompatible dimensions of routine arguments
*			    and internal variables. Probably indicates 
*			    a PARAMETER statement which is wrong in the
*			    routine.
*			2 - Insufficient internal storage in a routine
*			    to cope with size of arguments
***************************************************************************
      INTEGER ERRNO
      CHARACTER ROUTIN*8,SRCFIL*12
***************************************************************************

      IF (ERRNO.EQ.1) THEN
       WRITE(*,1) ROUTIN,SRCFIL
      ELSEIF (ERRNO.EQ.2) THEN
       WRITE(*,2) ROUTIN,SRCFIL
      ELSE
       WRITE(*,99) ROUTIN,SRCFIL
      ENDIF

      STOP

 1    FORMAT(/'*****ERROR***** In routine ',A8,' (source file ',
     +A12,'),',/,'some internal array dimensions are incompatible ',
     +'with the routine''s',/,'arguments. Check the PARAMETER ',
     +'statements in this routine, and recompile.')
 2    FORMAT(/'*****ERROR***** In routine ',A8,' (source file ',
     +A12,'),',/,'there is insufficient internal storage to cope ',
     +'with this problem.',/,'You should increase the dimension ',
     +'of some or all of the internal arrays for',
     +/,'this routine, and recompile.')
 99   FORMAT(/'*****ERROR***** Unidentified programming error ',
     +'found in routine ',A8,/,'(source file ',A12,')')
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE CORSET(FILNM,FILNO,SPMOD,RHO,CORMAT,NSITES,
     +			SITINF,NATTR,SCODES,MXN,MXP)
****************************************************************************
*	Sets up a spatial dependence matrix according to model SPMOD.
*	Arguments:
*	FILNM	(character, input). Name of file to read dependences
*		and distances from, if necessary.
*	FILNO	(integer, input). Channel to connect the file to.
*	SPMOD	(integer, input). Identifies which model to use for the
*		spatial dependence.
*	RHO	(double precision, input). Vector of spatial model 
*		parameters.
*	CORMAT	(double precision, output). Matrix of computed inter-site
*		dependences.
*	NSITES	(integer, input) No. of sites in simulation.
*	SITINF  (double precision, input). Array of site attributes. Used
*		to calculate distance-dependent dependences.
*       NATTR   (integer, input). Number of attributes defined in SITINF
*	SCODES 	(character, input). Array of short site codes.
*	MXN,MXP	(integer,input). Dimensioning.
****************************************************************************
      INTEGER FILNO,SPMOD,NSITES,NATTR,MXN,MXP
      DOUBLE PRECISION RHO(MXP),CORMAT(MXN,MXN),SITINF(MXN,MXP,0:3)
      CHARACTER FILNM*12,SCODES(MXN)*4
****************************************************************************
*	Additional INTEGERs
*	-------------------
*	I,J	counters
****************************************************************************
      INTEGER I,J
****************************************************************************
*	Additional DOUBLEs
*	------------------
*	CORR	Correlation read from input file
*       XSEP }  Inter-site separation in terms of the first two 
*       YSEP }  attributes defined in SITINF
*       DIST    Inter-site distance
****************************************************************************
      DOUBLE PRECISION CORR,XSEP,YSEP,DIST
****************************************************************************
*	Additional CHARACTERs
*	---------------------
*	SCODE1} Site codes read from input file
*	SCODE2}
****************************************************************************
      CHARACTER*4 SCODE1,SCODE2
*
*	'Independence' case. We treat it as a special case of the 
*	other models.
*
      IF (SPMOD.EQ.0) THEN
       DO 90 I=1,NSITES
        CORMAT(I,I) = 1.0D0
        DO 91 J=I+1,NSITES
         CORMAT(I,J) = 0.0D0
         CORMAT(J,I) = 0.0D0
 91     CONTINUE
 90    CONTINUE
*
*	Here's the `empirical correlation' case. We make sure all
*	the pairs of sites we're simulating for are represented in
*	the file we're reading from - initialise all correlations
*	to something impossible, then read in from file, and then 
*	pass through again to check that nothing impossible is left.
*       The value of -9.999 is the same as that used in the fitting
*       programs to indicate a pair of sites with no observations
*       in common. 
*
      ELSEIF (SPMOD.EQ.1) THEN
       CALL FILCHK(FILNM,FILNO,1)
       READ(FILNO,*)
       DO 100 I=1,NSITES
        CORMAT(I,I) = 1.0D0
        DO 101 J=I+1,NSITES
         CORMAT(I,J) = -9.999D0
         CORMAT(J,I) = -9.999D0
 101    CONTINUE
 100   CONTINUE
 
 110   READ(FILNO,'(A4,4X,A4,4X,F7.4)',END=199) SCODE1,SCODE2,CORR
       DO 120 I=1,NSITES
        DO 121 J=I+1,NSITES
         IF ( ((SCODE1.EQ.SCODES(I)).AND.(SCODE2.EQ.SCODES(J))).OR.
     +      ((SCODE1.EQ.SCODES(J)).AND.(SCODE2.EQ.SCODES(I))) ) THEN
          CORMAT(I,J) = CORR
          CORMAT(J,I) = CORR
          GOTO 110
         ENDIF
 121    CONTINUE
 120   CONTINUE
       GOTO 110

 199   CLOSE(FILNO)
       DO 130 I=1,NSITES
        DO 131 J=I+1,NSITES
         IF (DABS(CORMAT(I,J)).GT.1.0D0) THEN
          WRITE(*,10) SCODES(I),SCODES(J),FILNM
          STOP
         ENDIF
 131    CONTINUE
 130   CONTINUE
*
*	Here's the `constant dependence' case
*
      ELSEIF (SPMOD.EQ.2) THEN
       DO 200 I=1,NSITES
        CORMAT(I,I) = 1.0D0
        IF (DABS(RHO(1)).GT.1.0D0) THEN
         WRITE(*,11) FILNM,RHO(1)
         STOP
        ENDIF
        DO 201 J=I+1,NSITES
         CORMAT(I,J) = RHO(1)
         CORMAT(J,I) = RHO(1)
 201    CONTINUE
 200   CONTINUE
*
*       Exponential and powered exponential decay to zero or a threshold
*
      ELSEIF ((SPMOD.GE.3).AND.(SPMOD.LE.6)) THEN
       XSEP = 0.0D0
       YSEP = 0.0D0
       DO 220 I=1,NSITES
        CORMAT(I,I) = 1.0D0
        DO 221 J=I+1,NSITES
         IF (NATTR.GE.1) XSEP=SITINF(I,1,0)-SITINF(J,1,0)
         IF (NATTR.GE.2) YSEP=SITINF(I,2,0)-SITINF(J,2,0)
         DIST = DSQRT( (XSEP*XSEP) + (YSEP*YSEP) ) 
         IF (SPMOD.EQ.3) THEN
          CORMAT(I,J) = DEXP(-RHO(1)*DIST)
         ELSEIF (SPMOD.EQ.4) THEN
          IF (DABS(RHO(2)).GT.1.0D8) GOTO 990
          CORMAT(I,J) = RHO(2) + ( (1.0D0-RHO(2))*DEXP(-RHO(1)*DIST))
         ELSEIF (SPMOD.EQ.5) THEN
          IF (DABS(RHO(2)).GT.1.0D8) GOTO 990
          CORMAT(I,J) = DEXP(-RHO(1)*(DIST**RHO(2)))
         ELSEIF (SPMOD.EQ.6) THEN
          IF ( (DABS(RHO(2)).GT.1.0D8).OR.
     +                          (DABS(RHO(3)).GT.1.0D8) ) GOTO 990
          CORMAT(I,J) = RHO(3) + 
     +            ( (1.0D0-RHO(3)) * DEXP(-RHO(1)*(DIST**RHO(2))) )
         ENDIF         
         CORMAT(J,I) = CORMAT(I,J)
 221    CONTINUE
 220   CONTINUE
*
*	No other valid options defined at present
*
      ELSE
       CALL READERR(FILNM,SPMOD,46)
      ENDIF
      
      RETURN
      
 990  WRITE(*,12) FILNM
      STOP
      
 10   FORMAT('****ERROR**** I don''t have a valid spatial ',
     +'correlation between sites',
     +/A4, ' and ',A4,'. This is either because there is no ',
     +'entry in file ',A12,
     +/'for this site pair, or because the correlation in the ',
     +'file exceeds 1 in',
     +/'magnitude (which itself could be a missing value flag ',
     +'of -9.999).',/5X,'Please sort this out!')
 11   FORMAT('****ERROR**** In file ',A12,', you have asked for ',
     +'a constant spatial',
     +/'correlation whose magnitude is greater than 1!')
 12   FORMAT('****ERROR**** In file ',A12,', you haven''t defined ',
     +'all the parameters',
     +/'required for your chosen correlation structure')
      END
