      PROGRAM SIMRAIN
******************************************************************************
*       PROGRAM AUTHOR:         RICHARD E. CHANDLER
*                               (email: richard@stats.ucl.ac.uk)
*       AFFILIATION:            DEPARTMENT OF STATISTICAL SCIENCE
*                                UNIVERSITY COLLEGE LONDON
*                                 GOWER STREET
*                                  LONDON WC1E 6BT
*                                   UK
*       LAST MODIFIED: November 2010
*        UCL 2001 - 2010
******************************************************************************
******************************************************************************
******************************************************************************
*       DESCRIPTION
*       ===========
*       This program is designed to simulate daily rainfall values at a
*       network of sites. It uses two generalised linear models to 
*       determine the marginal probability distributions of rainfall amounts
*       at individual sites (the first determines the probability that a site
*       is wet or dry and the second, applied only to wet sites, determines
*       the probability density function for the amount of rain). Spatial
*       association is then built in by simulating correlated model residuals
*       and transforming back to the appropriate distribution at each site.
*
*       The following classes of explanatory variable are allowed for:
*       -       Previous days' rainfall behaviour (up to 3 previous days)
*       -       Seasonal effects, using sinusoids fitted by month
*       -       Long-term trends (climate change)
*       -       Site effects e.g. elevation
*       -       `External' climatological factors
*       In addition, 2- and 3-way interactions between components may be
*       specified.
*
*       The program should be compiled along with the following other source
*       files:
*       
*       randgen.f       Contains code for random number generation
*       raingen.f       Generates a day's rainfall at a user-specified
*                       network of sites, some of whose rainfall amounts
*                       may be known a priori
*       setpred.f       Computes components of linear predictors
*       funcdefs.f      Different scenarios for climate change, seasonal
*                       patterns, dependence on previous days etc.
*       sitexfrm.f      Transforms attributes of different sites to
*                       allow for nonlinear dependence
*       rec_math.f      Various mathematical routines and functions
*
*	REVISION HISTORY: see manual
******************************************************************************
******************************************************************************
******************************************************************************
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*>>>>>>>>>>>>>>>>>>>>>>>>>>>    PARAMETERS      >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*       MXNSITES        Maximum no. of sites the program will cope with. There
*                       is a corresponding statement in almost every 
*                       subroutine.
*       MXPARAMS        Maximum number of parameters in each model. NB IF
*                       THIS IS CHANGED, THE CORRESPONDING ENTRIES IN
*                       BLOCK DATA UNIT DSCTXT (FILE FUNCDEFS.F) WILL
*                       ALSO NEED CHANGING. ALSO SEE MOST ROUTINES IN
*                       FILE RAINGEN.F.
*	MXNP		MXPARAMS * MXNSITES. Used solely for DATA statements;
*			its necessity is a consequence of the Flexibility of
*			Fortran ...
******************************************************************************
	INTEGER MXNSITES,MXPARAMS,MXNP
	PARAMETER (MXNSITES=200,MXPARAMS=80,MXNP=MXNSITES*MXPARAMS)
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*>>>>>>>>>>>>>>>>>>>>>>>>>>>      VARIABLES     >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
***************************************************************************
*       INTEGER variables
*       ^^^^^^^^^^^^^^^^^
*       NSITES  - no. of sites required (i.e. index of last entry in RAIN)
*       NREGS   - no. of regions used
*       NATTR   - no. of site attributes (e.g. elevation, Eastings etc.)
*       FLAGS   - array indicating required status of each entry of RAIN:
*                 0 = Rainfall amount known, don't change it
*                 1 = Rainfall amount unknown, simulate from the model
*	MISSFL	- Flags indicating whether or not covariates aare missing at each site
*		  for a particular day. Required as an argument to COVSET.
*       DATINIT - Indicator for status of data initialisation:
*                 0 = nothing initialised yet
*                 1 = found data in file, and we're initialising from that
*                     as far as possible
*                 2 = we're past the end of the input file
*	FORCE	- flag to indicate whether to force recalculation of all 
*	   	  covariates (1) or just those which have changed since last
*		  calculation (0)
*       YY      - current year
*       YRLIMS  - first and last years to do
*       MM      - current month
*       MOLIMS  - first and last months to do
*       DD      - current day
*       DAYS_IN_MONTH   - no. of days in current month (integer function)
*       DATE    - current month, in YYYYMM format
*       DATELIMS- start and end months in YYYYMM format
*       NSIMS   - number of simulations
*       SIMNO   - number of current simulation
*       SEED    - seed for random number generator
*       OUTOPT  - selects output options:
*                 1 = Output summary information only (monthly and annual 
*                     totals, averaged over the whole region and/or various
*                     subregions)
*                 2 = Write an output file with 1 record for every day in 
*                     the simulation, containing daily values for each site.
*                     Do not produce summary measures
*                 3 = Do both.
*       OUTSIM  - Number of realisation for which daily output is required
*       OUTYR   - Start and end years for which daily output is required
*       OUTMO   - Start and end months for which daily output is required
*       REGCODE - Region codes for each site
*       WHICHREGS- Chooses which regions we want to generate summary statistics
*                  for. Element 0 is the whole area
*       NP1     - Numbers of parameters in logistic regression model. Elements
*                 are:
*                 1 - No. of main effects associated with site
*                 2 - NP1(1) + No. of main effects associated with year
*                 3 - NP1(2) + No. of main effects associated with month
*                 4 - NP1(3) + No. of main effects associated with previous
*                               days' values
*                 5 - NP1(4) + No. of 2-way interactions
*                 6 - NP1(5) + No. of 3-way interactions
*		  7 - NP1(6) + No. of nonlinear parameters
*                 8 - No. of `globa;' parameters such as trace thresholds
*		  9 - No. of dispersion parameters
*		 10 - No. of spatial correlation parameters
*       NP2     - Numbers of parameters in model for intensities. Elements as
*                 in NP1
*       COVCODE1 - Code numbers of covariates in logistic regression model
*       COVCODE2 - Code numbers of covariates in intensity model
*       GLBCD1   - Definitions of `global' quantities in occurrence model
*       GLBCD2   - Definitions of `global' quantities in amounts model
*	FOUID1 }- Index arrays for Fourier representation of site effects. 
*	FOUID2 }  See file funcdefs.f for full details
*	LEGID1 }- Index arrays for Legendre polynomial representation of
*	LEGID2 }  site effects.
*       PWTID1 }- Indexing for weighted averages of previous days' values
*       PWTID2 }  ((I,0)th entry is the number of the covariate to which the
*                 parametrisation for the Ith weighting scheme is attached,
*                 (I,J)th is the model number of the Jth parameter in the
*                 scheme).
*       WTSCH1 }- Indexing for weighted average of previous days' values
*       WTSCH2 }  (Ith element is the weighting scheme used in Ith
*                 covariate).
*	MXLAG1 } - Argument to MDLSET routine. Used in the interests of
*	MXLAG2 }   efficiency - only keep track of last MXLAG? days
*	SPMOD1	 - Spatial correlation model used for occurrence
*	SPMOD2	 - Ditto, intensity
*       IEXTD } - Indicate whether `external' predictors at daily,
*       IEXTM }   monthly or yearly timescales have been requested
*       IEXTY }
*	NLSTAT   - Array used for compatibility with MDLSET routines - not 
*		   needed here except as a dummy argument.
*	SITXFM1	} Select nonlinear transformations of site attributes in
*	SITXFM2 } each model. First column chooses the attribute, second 
*		  the transformation
*       TWO1    - indices of 2-way interactions in logistic regression model
*       TWO2    - ditto, intensity model
*       THREE1  - guess ...
*       THREE2  -    "
*	PUPDT	- indicates which components of the linear predictors are
*		  to be updated. If PUPDT = 1, all components are updated;
*		  if 2, year, seasonal and daily effects, if 3, just
*		  seasonal and daily and if 4 just daily
*       THRTYP  - Type of thresholding to be carried out on small
*                 positive values. 0: no thresholding; 1: `soft'
*                 thresholding (zero everything below threshold, 
*                 subtract threshold from everything above); 2:
*                 `hard' thresholding (just zero everything below
*                 threshold)
*       ICHECK  - Indicates whether model parametrisations have been 
*                 checked
***************************************************************************
      INTEGER NSITES,NREGS,FLAGS(MXNSITES),MISSFL(MXNSITES)
      INTEGER NATTR
      INTEGER YY,YRLIMS(2),MM,MOLIMS(2),DD,DAYS_IN_MONTH
      INTEGER DATE,DATELIMS(2),NSIMS,SIMNO
      INTEGER DATINIT,FORCE
      INTEGER SEED,OUTOPT,OUTSIM,OUTYR(2),OUTMO(2)
      INTEGER REGCODE(MXNSITES),WHICHREGS(0:MXNSITES)
      INTEGER NP1(10),NP2(10)
      INTEGER COVCODE1(MXPARAMS),COVCODE2(MXPARAMS)
      INTEGER GLBCD1(MXPARAMS),GLBCD2(MXPARAMS)
      INTEGER FOUID1(MXPARAMS),FOUID2(MXPARAMS)
      INTEGER LEGID1(MXPARAMS),LEGID2(MXPARAMS)
      INTEGER PWTID1(MXPARAMS,0:3),WTSCH1(MXPARAMS)
      INTEGER PWTID2(MXPARAMS,0:3),WTSCH2(MXPARAMS)
      INTEGER MXLAG1,MXLAG2,SPMOD1,SPMOD2
      INTEGER IEXTD,IEXTM,IEXTY
      INTEGER NLSTAT(MXPARAMS,3)
      INTEGER SITXFM1(MXPARAMS,2),SITXFM2(MXPARAMS,2)
      INTEGER TWO1(MXPARAMS,2),TWO2(MXPARAMS,2)
      INTEGER THREE1(MXPARAMS,3),THREE2(MXPARAMS,3)
      INTEGER PUPDT,THRTYP,ICHECK
      INTEGER I,J
***************************************************************************
*       DOUBLE PRECISION variables
*       ^^^^^^^^^^^^^^^^^^^^^^^^^^
*       RAIN    - vector of daily rainfall amounts
*       LOGITS  - vector of modelled logits for a particular day
*       MEANS   - vector of modelled rainfall means for a particular day
*       NU      - vector of gamma shape parameters for the rainfall intensities
*       ANSMEAN - vector of mean Anscombe residuals for intensity model
*	CORMAT1	- Inter-site latent correlations for logistic model
*	CORMAT2	- Ditto, intensity model (Anscombe correlations)
*       PREVIOUS- Array of previous days' rainfall amounts: Jth column
*                 corresponds to J days previously
*       SITINF- Information regarding each site (Eastings, Northings etc.)
*       GLBVL?   - Values of `global' quantities. These are specified
*		  in 2 input files, so we have 2 values and check that they're
*		  the same.
*       MOTOT   - Monthly rainfall totals at each gauge
*       YRTOT   - Annual rainfall totals at each gauge
*       PRED$   - components of linear predictors. $=1 corresponds to
*                  the logistic regression part of the model, $=2 to
*                  the rainfall intensity part. The elements indicate which
*                  set of explanatory variables is represented:
*                       1 - all the bits we know once the site is fixed
*                       2 - the additional bits once site and year are fixed
*                       3 - site, year and month
*                       4 - site, year, month and previous day
*       BETA1   - parameters in logistic regression model. Position 0 is
*                 for constant term
*       BETA2   - parameters in intensity model
*	THETA1	- parameters in nonlinear transformations, logistic model.
*	THETA2	- ditto, intensity model
*	RHO1	- parameters defining spatial correlation matrix, logistic
*	RHO2	- ditto, intensity model
*       COVS1   - Values of covariates in logistic regression model
*       COVS2   - Values of covariates in intensity model
*       THRESH  - Threshold for small values
*       TRACE   - Ditto, when we're treating them as `trace' values
******************************************************************************
      DOUBLE PRECISION RAIN(MXNSITES)
      DOUBLE PRECISION LOGITS(MXNSITES),MEANS(MXNSITES),NU(MXNSITES)
      DOUBLE PRECISION ANSMEAN(MXNSITES)
      DOUBLE PRECISION CORMAT1(MXNSITES,MXNSITES)
      DOUBLE PRECISION CORMAT2(MXNSITES,MXNSITES)
      DOUBLE PRECISION PREVIOUS(MXNSITES,10)
      DOUBLE PRECISION SITINF(MXNSITES,MXPARAMS,0:3)
      DOUBLE PRECISION GLBVL1(MXPARAMS),GLBVL2(MXPARAMS)
      DOUBLE PRECISION MOTOT(MXNSITES,12),YRTOT(MXNSITES)
      DOUBLE PRECISION PRED1(MXNSITES,4),PRED2(MXNSITES,4)
      DOUBLE PRECISION BETA1(0:MXPARAMS),BETA2(0:MXPARAMS)
      DOUBLE PRECISION THETA1(MXPARAMS,3),THETA2(MXPARAMS,3)
      DOUBLE PRECISION RHO1(MXPARAMS),RHO2(MXPARAMS)
      DOUBLE PRECISION COVS1(MXNSITES,MXPARAMS),COVS2(MXNSITES,MXPARAMS)
      DOUBLE PRECISION THRESH,TRACE
***************************************************************************
*       CHARACTER variables
*       ^^^^^^^^^^^^^^^^^^^
*       CHAN30  - name of input file containing data from gauges
*       CHAN31  - name of input file containing external annual predictors
*       CHAN32  - name of input file containing external monthly predictors
*       CHAN33  - name of input file containing external daily predictors
*       CHAN35  - name of input file containing description of subareas
*       CHAN36  - name of input file containing site definitions
*       CHAN41  - name of input file containing specification for logistic
*                 model
*       CHAN42  - name of input file containing specfication for intensity
*                 model
*	CHAN66	- name of input file containing empirical correlations
*		  (wet/dry model)
*	CHAN67	- ditto (intensity model)
*       CHAN50  - name of output file containing daily simulation results
*       CHAN51  - name of output file containing monthly summaries
*       CHAN60  - name of output file containing model description
*       TITLE1  - some meaningful description of the logistic model
*       TITLE2  - ditto, intensity model
*       MDLTXT1 - array of descriptors for model covariates (logistic)
*       MDLTXT2 - array of descriptors for model covariates (intensity)
*	NLTXT1	- array of descriptors for nonlinear parameters (logistic)
*	NLTXT2	- array of descriptors for nonlinear parameters (intensity)
*       REGTXT  - array of subarea descriptors
*       SITETXT - array of site descriptors etc.
*       SCODES  - array of short site codes
*       ATTRTXT - Text for site attributes
*       MOTXT   - Text for seasonal model components
*       TRNDTXT - Text for trend components
*       DYTXT   - Text for daily components
*       PWTTXT  - Text for parameters in nonlinear weighting schemes for
*                 averaging previous days' values
*	TRPTXT	- Text for nonlinear trend transformations
*	SXPTXT	- Text for nonlinear site transformations
*	SPATXT	- Text for spatial correlation models
*	SPPTXT	- Text for parameters in spatial correlation 
*		  model (dimension to MXPARAMS for convenience)
*       CORTXT  - array describing quantities to which inter-site 
*                 correlations correspond
*       GLBTXT  - Text for global quantities
*       DLYFMT  - Output format for daily data (depends on the number
*                 of sites)
***************************************************************************
      CHARACTER*12 CHAN30,CHAN31,CHAN32,CHAN33,CHAN35,CHAN36
      CHARACTER*12 CHAN41,CHAN42
      CHARACTER*12 CHAN50,CHAN51,CHAN60,CHAN66,CHAN67
      CHARACTER*70 TITLE1,TITLE2
      CHARACTER*70 MDLTXT1(0:MXPARAMS),MDLTXT2(0:MXPARAMS)
      CHARACTER*70 NLTXT1(MXPARAMS,3),NLTXT2(MXPARAMS,3)
      CHARACTER REGTXT(0:MXNSITES)*60,SITETXT(MXNSITES)*70
      CHARACTER SCODES(MXNSITES)*4
      CHARACTER*70 ATTRTXT(MXPARAMS),GLBTXT(MXPARAMS)
      CHARACTER*70 PWTTXT(MXPARAMS,3),MOTXT(MXPARAMS)
      CHARACTER*70 TRNDTXT(MXPARAMS),DYTXT(MXPARAMS)
      CHARACTER*70 TRPTXT(MXPARAMS,3),SXPTXT(MXPARAMS,3)
      CHARACTER*70 SPATXT(0:MXPARAMS),SPPTXT(MXPARAMS,4)
      CHARACTER*70 CORTXT(MXPARAMS)
      CHARACTER*43 DLYFMT
***************************************************************************
*       LOGICAL variables
*       ^^^^^^^^^^^^^^^^^
*       FILOPN  TRUE if data file is open, FALSE otherwise. Used to
*               determine whether we need to close it.
***************************************************************************
      LOGICAL FILOPN
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*>>>>>>>>>>>>>>>>>>>>>>>>>>>    FILE NUMBERS    >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*       The following FORTRAN file numbers are allocated in this program:
*       UNIT    DESCRIPTION
*       ----    -----------
*       30      Input file containing data from gauges
*       31      Input file containing external annual predictors
*       32      Input file containing external monthly predictors
*       33      Input file containing external daily predictors
*       35      Input file containing definition of subareas
*       36      Input file containing definition of sites
*       41      Input file containing logistic model definition
*       41      Input file containing gamma model definition
*       50      Output file, containing 1 record for each day in the simulation
*       51      Output file, containing monthly and annual summary information
*               regarding the simulation
*       60      Output file, containing model description
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*>>>>>>>>>>>>>>>>>>>>>>>>>>>    LINE NUMBERS    >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*       The following line numbering convention is used:
*       LABEL RANGE     DESCRIPTION
*       -----------     -----------
*       1 - 99          FORMAT statements
*       500 - 599       Daily simulation        
*       600-699         Multiple simulations
*       900-999         Error trapping
***************************************************************************
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*>>>>>>>>>>>>>>>>>>>>>>>>>>>    COMMON BLOCKS    >>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
*       DESCRIBE        Descriptive text
**************************************************************************
      COMMON /DESCRIBE/ SXPTXT,MOTXT,TRNDTXT,TRPTXT,DYTXT,
     +			SPATXT,SPPTXT,CORTXT,PWTTXT,GLBTXT
*
*       Initializing ...
*
      DATA (RAIN(I),I=1,MXNSITES) /MXNSITES*-1.0D0/
      DATA (FLAGS(I),I=1,MXNSITES) /MXNSITES*1/
      DATA ((COVS1(I,J),I=1,MXNSITES),J=1,MXPARAMS) /MXNP*0.0D0/
      DATA ((COVS2(I,J),I=1,MXNSITES),J=1,MXPARAMS) /MXNP*0.0D0/
      DATA (NP1(I),NP2(I),I=1,10),MXLAG1,MXLAG2 /22*0/
      DATA (BETA1(I),BETA2(I),I=0,MXPARAMS) /
     +MXPARAMS*0.0D0,MXPARAMS*0.0D0,2*0.0D0 /
      DATA ((THETA1(I,J),THETA2(I,J),I=1,MXPARAMS),J=1,3) /
     +MXPARAMS*1.0D9,MXPARAMS*1.0D9,MXPARAMS*1.0D9,MXPARAMS*1.0D9,
     +MXPARAMS*1.0D9,MXPARAMS*1.0D9 /
      DATA (GLBVL1(I),GLBVL2(I),I=1,MXPARAMS) /
     +MXPARAMS*1.0D9,MXPARAMS*1.0D9 /
      DATA (GLBCD1(I),GLBCD2(I),I=1,MXPARAMS) /
     +MXPARAMS*-1,MXPARAMS*-1 /
      DATA ((SITXFM1(I,J),SITXFM2(I,J),I=1,MXPARAMS),J=1,2) 
     +/MXPARAMS*0,MXPARAMS*0,MXPARAMS*0,MXPARAMS*0/
      DATA (FOUID1(I),I=1,MXPARAMS) /MXPARAMS*0/
      DATA (FOUID2(I),I=1,MXPARAMS) /MXPARAMS*0/
      DATA (LEGID1(I),I=1,MXPARAMS) /MXPARAMS*0/
      DATA (LEGID2(I),I=1,MXPARAMS) /MXPARAMS*0/
      DATA ((PWTID1(I,J),J=0,3),I=1,MXPARAMS) /
     +MXPARAMS*0,MXPARAMS*0,MXPARAMS*0,MXPARAMS*0/
      DATA ((PWTID2(I,J),J=0,3),I=1,MXPARAMS) /
     +MXPARAMS*0,MXPARAMS*0,MXPARAMS*0,MXPARAMS*0/
      DATA (WTSCH1(I),I=1,MXPARAMS) /MXPARAMS*0/
      DATA (WTSCH2(I),I=1,MXPARAMS) /MXPARAMS*0/
      DATA ((NLSTAT(I,J),I=1,MXPARAMS),J=1,3)
     +/MXPARAMS*-1,MXPARAMS*-1,MXPARAMS*-1/
      DATA SPMOD1,SPMOD2,(RHO1(I),RHO2(I),I=1,MXPARAMS)
     +/0,0,MXPARAMS*1.0D9,MXPARAMS*1.0D9/
      DATA (WHICHREGS(I),I=0,MXNSITES) /1,MXNSITES*0/
      DATA IEXTD,IEXTM,IEXTY,ICHECK/4*0/
  
      DATA CHAN30 /'gaugvals.dat'/
      DATA CHAN31,CHAN32,CHAN33 /'yr_preds.dat','mn_preds.dat',
     +'dy_preds.dat'/
      DATA CHAN35,CHAN36 /'regions.def','siteinfo.def'/
      DATA CHAN41,CHAN42 /'logistic.def','gammamdl.def'/
      DATA CHAN50,CHAN51 /'daily.dat','monthly.dat'/
      DATA CHAN60 /'mdlspec.txt'/
      DATA CHAN66,CHAN67 /'cor_logi.dat','cor_gamm.dat'/
*
*       Open files for data, model specification and output (although
*	we're not guaranteed to use all output files, it's best to do
*	all these checks before we've spent ages inputting stuff rather
*	than get half-way down and then discover there's a screwup!)
*
      WRITE(*,*) 'Opening files ...'
      CALL FILCHK(CHAN30,30,1)
      CALL FILCHK(CHAN35,35,1)
      CALL FILCHK(CHAN36,36,1)
      CALL FILCHK(CHAN41,41,1)
      CALL FILCHK(CHAN42,42,1)
      CALL FILCHK(CHAN50,50,2)
      CALL FILCHK(CHAN51,51,2)
      CALL FILCHK(CHAN60,60,2)
*
*       Read definitions of subareas and site locations, and check
*
      CALL REGDEF(CHAN35,35,REGTXT,NREGS,MXNSITES)
      CALL CHKREG(REGTXT,NREGS,MXNSITES)
      CALL SITEDEF(36,CHAN36,ATTRTXT,NATTR,SITINF,REGCODE,
     +                    SCODES,SITETXT,NSITES,MXNSITES,MXPARAMS)
      CALL CHKSIT(REGCODE,SCODES,SITETXT,NSITES,MXNSITES,MXPARAMS)
*
*     Define record format for daily output data
*
      IF (NSITES.LT.100000) THEN 
       WRITE(DLYFMT,'(A22,I5,A16)') '(I3,1X,I4,1X,2(I2,1X),',
     +                                   NSITES,'(F6.2,1X,I1,1X))'
      ELSE
       GOTO 910
      ENDIF
      WRITE(60,'(//,''MODEL DEFINITIONS:'',/,18(''~''),/)')
*
*       Read model specification, output to screen and summary
*       file, and check it's OK. Calculate nonlinear transformations
*	of site attributes, and output the descriptive text
*       associated with each model choice for further checking. NB
*	the routine CHKMDL doubles as a `model' output routine in 
*	fitting programs, where standard errors of parameters are
*	involved - here, we use any convenient arrays lying around
*	as dummies. Having defined the model, we calculate spatial
*	correlation matrices (covariances for intensity model)
*
      CALL MDLSET(41,CHAN41,MXPARAMS,1,0,NATTR,NP1,GLBVL1,BETA1,
     +	THETA1,NU(1),COVCODE1,GLBCD1,MXLAG1,SPMOD1,IEXTD,IEXTM,
     +  IEXTY,SITXFM1,WTSCH1,TWO1,THREE1,NLSTAT,
     +  FOUID1,LEGID1,PWTID1,SITINF,MXNSITES,NSITES,RHO1,TITLE1)
      CALL MDLSET(42,CHAN42,MXPARAMS,11,1,NATTR,NP2,GLBVL2,BETA2,
     +	THETA2,NU(1),COVCODE2,GLBCD2,MXLAG2,SPMOD2,IEXTD,IEXTM,
     +  IEXTY,SITXFM2,WTSCH2,TWO2,THREE2,NLSTAT,
     +  FOUID2,LEGID2,PWTID2,SITINF,MXNSITES,NSITES,RHO2,TITLE2)
      IF (IEXTY.EQ.1) CALL FILCHK(CHAN31,31,1)
      IF (IEXTM.EQ.1) CALL FILCHK(CHAN32,32,1)
      IF (IEXTD.EQ.1) CALL FILCHK(CHAN33,33,1)
      IF (NP1(8).NE.NP2(8)) GOTO 900
      DO 100 I=1,NP1(8)
       IF (GLBCD1(I).NE.GLBCD2(I)) GOTO 900
       IF (DABS(GLBVL1(GLBCD1(I)/1000)-GLBVL2(GLBCD2(I)/1000))
     +                                            .GE.1.0D-6) GOTO 900
 100  CONTINUE
      CALL ATTRXFM(TITLE1,SITINF,NSITES,ATTRTXT,NATTR,COVCODE1,
     +				SITXFM1,FOUID1,LEGID1,THETA1,NP1,
     +				ICHECK,MXNSITES,MXPARAMS)
      CALL TXTSET(NP1,COVCODE1,GLBCD1,SITXFM1,PWTID1,WTSCH1,ATTRTXT,
     +       SXPTXT,TRNDTXT,TRPTXT,MOTXT,DYTXT,PWTTXT,MDLTXT1,NLTXT1,
     +       GLBTXT,MXPARAMS)
      CALL CHKMDL(BETA1,BETA1,THETA1,THETA1,GLBVL1,-1.0D0,RHO1,COVCODE1,
     +	TWO1,THREE1,GLBCD1,NP1,0,PWTID1,SPMOD1,TITLE1,MDLTXT1,NLTXT1,
     +  SPATXT,SPPTXT,CORTXT,ATTRTXT,GLBTXT,1,1,MXPARAMS)
      WRITE(60,*)
      CALL ATTRXFM(TITLE2,SITINF,NSITES,ATTRTXT,NATTR,COVCODE2,
     +				SITXFM2,FOUID2,LEGID2,THETA2,NP2,
     +				ICHECK,MXNSITES,MXPARAMS)
      CALL TXTSET(NP2,COVCODE2,GLBCD2,SITXFM2,PWTID2,WTSCH2,ATTRTXT,
     +        SXPTXT,TRNDTXT,TRPTXT,MOTXT,DYTXT,PWTTXT,MDLTXT2,NLTXT2,
     +        GLBTXT,MXPARAMS)
      CALL CHKMDL(BETA2,BETA2,THETA2,THETA2,GLBVL2,NU(1),RHO2,COVCODE2,
     +	TWO2,THREE2,GLBCD2,NP2,0,PWTID2,SPMOD2,TITLE2,MDLTXT2,NLTXT2,
     +  SPATXT,SPPTXT,CORTXT,ATTRTXT,GLBTXT,1,11,MXPARAMS)
      WRITE(60,*)
      CALL RHOCHK(RHO1,SPMOD1,CHAN41,MXPARAMS)
      IF ((SPMOD1.GT.0).AND.(SPMOD1.LT.20)) THEN 
       CALL CORSET(CHAN66,66,SPMOD1,RHO1,CORMAT1,NSITES,SITINF,
     +                              NATTR,SCODES,MXNSITES,MXPARAMS)
      ENDIF
      CALL RHOCHK(RHO2,SPMOD2,CHAN42,MXPARAMS)
      CALL CORSET(CHAN67,67,SPMOD2,RHO2,CORMAT2,NSITES,SITINF,
     +				    NATTR,SCODES,MXNSITES,MXPARAMS)
      CALL ANSET(ANSMEAN,CORMAT2,NU(1),NSITES,MXNSITES,MXPARAMS)
      IF (MXLAG1.LT.MXLAG2) MXLAG1 = MXLAG2
*
*       Now define start and end dates, and number of simulations etc.
*
      CALL SIMSET(YRLIMS,MOLIMS,NSIMS,OUTOPT,60,
     +            REGTXT,NREGS,WHICHREGS,OUTSIM,OUTYR,OUTMO,MXNSITES)
*
*       Set miscellanea - random number seed.
*
      CALL MSCSET(SEED)
*
*       Stop writing to model specification file - we've finished
*       defining everything
*
      CLOSE(60)
*
*     Define thresholds for small positive values, if required
*
      CALL TAUDEF(GLBVL1,GLBCD1,NP1(8),THRESH,TRACE,THRTYP,MXPARAMS)
*
*       Calculate start and end months in YYYYMM format
*
      DATELIMS(1) = (100*YRLIMS(1))+MOLIMS(1)
      DATELIMS(2) = (100*YRLIMS(2))+MOLIMS(2)
*
*       Simulation/interpolation starts here. Loop over years, months 
*	and days of simulation. Only need to compute site contributions
*	to predictors once, so set PUPDT = 1 here.
*
      PUPDT = 1
      DO 600 SIMNO=1,NSIMS
*
*       Reinitialise everything. Also copy NU(1) into other elements 
*       of NU. In the future we may want to allow NU to vary between
*       sites, but for the moment this seems as good a place as any 
*       to deal with it.
*
      DO 610 I=1,NSITES
       RAIN(I) = 0.0D0
       NU(I) = NU(1)
       DO 611 J=1,10
	PREVIOUS(I,J) = 0.0D0
 611   CONTINUE
 610  CONTINUE
      IF (SIMNO.EQ.1) THEN
       DATINIT = -1
      ELSE
       DATINIT = 0
      ENDIF

      DO 500 YY = YRLIMS(1),YRLIMS(2)
       DO 510 I=1,NSITES
	YRTOT(I) = 0.0D0
 510   CONTINUE

*
*       A new year - need to update trend predictors (unless we're
*	already calculating site predictors)
*
       IF (PUPDT.GE.2) PUPDT = 2
       DO 520 MM = 1,12
	DO 530 I=1,NSITES
	 MOTOT(I,MM) = 0.0D0
 530    CONTINUE
*
*       Skip if we're outside requested range
*
	DATE = (100*YY) + MM
	IF ( (DATE.LT.DATELIMS(1)).OR.(DATE.GT.DATELIMS(2)) ) GOTO 520

	WRITE(*,10) MM,YY,SIMNO,NSIMS
*
*       New month - need to update seasonal predictors (unless we're already 
*	doing trend predictors). Also FORCE recalculation of all
*       covariates
*
        IF (PUPDT.GE.3) PUPDT = 3
        FORCE = 1
	DO 540 DD = 1,DAYS_IN_MONTH(MM,YY)
*
*       First thing is to update the data values, FLAGS etc. Exactly
*       what we do is dependent on the status of DATINIT
*
 	 CALL UPDAT2(RAIN,PREVIOUS,THRESH,THRTYP,FLAGS,NSITES,
     +           YY,MM,DD,MXLAG1,DATINIT,MXNSITES,CHAN30,30,SCODES)
*
*     Calculate linear predictors, and set FORCE to zero (initially it's 1).
*     Also set PUPDT to 4 so that next time through we'll just update
*     day-to-day quantities. NB use of MXLAG1 both times is CORRECT!
*
         CALL COVSET(COVS1,NP1,NSITES,COVCODE1,TWO1,THREE1,GLBCD1,
     +	     SITINF,YY,MM,DD,PREVIOUS,PWTID1,MXLAG1,GLBVL1,BETA1,
     +       THETA1,TRACE,0,FORCE,MISSFL,ICHECK,MXNSITES,MXPARAMS,1,1)
	 CALL SETPRED(PRED1,COVS1,NP1,NSITES,COVCODE1,
     +     TWO1,THREE1,BETA1,PUPDT,MXNSITES,MXPARAMS)
         CALL COVSET(COVS2,NP2,NSITES,COVCODE2,TWO2,THREE2,GLBCD2,
     +	     SITINF,YY,MM,DD,PREVIOUS,PWTID2,MXLAG1,GLBVL2,BETA2,
     +	     THETA2,TRACE,0,FORCE,MISSFL,ICHECK,MXNSITES,MXPARAMS,1,1)
	 CALL SETPRED(PRED2,COVS2,NP2,NSITES,COVCODE2,
     +     TWO2,THREE2,BETA2,PUPDT,MXNSITES,MXPARAMS)
         FORCE = 0
         PUPDT = 4
*
*     If we got through that lot, model parametrisation must be OK -
*     no further need to check
*
         ICHECK = 1
*
*       Calculate predicted logits and means
*
	 DO 550 I=1,NSITES
	  LOGITS(I) = PRED1(I,4)
	  MEANS(I) = DEXP(PRED2(I,4))
 550     CONTINUE
*
*       Calculate the vector of rainfall amounts for today
*
	 CALL RAINGEN(RAIN,FLAGS,NSITES,SPMOD1,LOGITS,RHO1,CORMAT1,
     +			SPMOD2,MEANS,NU,CORMAT2,ANSMEAN,MXNSITES)
*
*     If the models were fitted to *soft thresholded* data, we need to add
*     the threshold back in to non-zero amounts. Hard-thresholded data
*     is left as simulated.
*
         IF (THRTYP.EQ.1) THEN
          DO 555 I = 1,NSITES
           IF (RAIN(I).GT.0.0D0) RAIN(I) = RAIN(I) + THRESH
 555      CONTINUE
         ENDIF
*
*       Depending on output option selected, EITHER update summary stats for
*       each gauge for this month, OR write the day's results to the output
*       file (if within the range requested), OR both
*
	 IF ( (OUTOPT.EQ.1).OR.(OUTOPT.EQ.3) ) THEN
	  DO 560 I=1,NSITES
	   MOTOT(I,MM) = MOTOT(I,MM) + RAIN(I)
 560      CONTINUE
	 ENDIF
	 IF ( (OUTOPT.EQ.2).OR.(OUTOPT.EQ.3) ) THEN
	  IF ( (OUTSIM.EQ.0).OR.
     +         ( (OUTSIM.EQ.SIMNO).AND.
     +           (100*(YY-OUTYR(1))+MM-OUTMO(1).GE.0).AND.
     +           (100*(YY-OUTYR(2))+MM-OUTMO(2).LE.0) ) ) THEN
	   WRITE(50,DLYFMT) SIMNO,YY,MM,DD,(RAIN(I),FLAGS(I),I=1,NSITES)
	  ENDIF
	 ENDIF

*
*     Now: if models were fitted to *soft thresholded* data, we need to
*     re-apply the threshold so that, when today's values become
*     covariates for tomorrow's distributions, they mean the same as
*     they did when the models were fitted. At this point there won't
*     be any positive values less than the threshold (see line 555 above)
*     so we don't need to worry about them.
*
         IF (THRTYP.EQ.1) THEN
          DO 565 I = 1,NSITES
           IF (RAIN(I).GT.0.0D0) RAIN(I) = RAIN(I) - THRESH
 565      CONTINUE
         ENDIF
	 
 540    CONTINUE
*
*       Add monthly totals to yearly ones if required
*
	IF ( (OUTOPT.EQ.1).OR.(OUTOPT.EQ.3) ) THEN
	 DO 535 I=1,NSITES
	  YRTOT(I) = YRTOT(I) + MOTOT(I,MM)
 535     CONTINUE
	ENDIF
 520   CONTINUE
*
*       Write this year's summary stats to output file if required
*
       IF ( (OUTOPT.EQ.1).OR.(OUTOPT.EQ.3) ) THEN
	CALL OUTSUM(SIMNO,YY,YRTOT,MOTOT,NSITES,REGCODE,WHICHREGS,
     +                                          NREGS,MXNSITES)
       ENDIF
 500  CONTINUE

*
*       That simulation is over - close data file if it's open.
*
      INQUIRE(UNIT=30,OPENED=FILOPN)
      IF (FILOPN) REWIND(30)

 600  CONTINUE

      CLOSE(50)
      CLOSE(51)

      STOP
*
*	Error trapping below here
*
 900  WRITE(*,20) CHAN41,CHAN42
      STOP
 910  WRITE(*,50)
      STOP
 
 10   FORMAT(5X,'Current month: ',I2,'/',I4,' (realisation ',I4,
     +' of ',I4,')')
 20   FORMAT(//10X,'****ERROR**** Files ',A12,' and ',A12,' contain',
     +/10X,'different definitions of `global'' quantities.')
 50   FORMAT(//10X,'****ERROR**** You have defined more than ',
     +'99,999 sites. ',/10X,'To deal with this much output, you ',
     +'will need to redefine the variable',/10X,'DLYFMT in file ',
     +'simrain.f, and recompile.')
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE REGDEF(FILNM,FILNO,REGTXT,NREGS,MXN)
*
*       To define numbers of subareas, and their names. Descriptive text 
*       goes into REGTXT. NREGS is the number of subareas (regions), 
*       FILNM is name of definition file, attached to unit FILNO
*
      INTEGER NREGS,MXN,FILNO
      CHARACTER REGTXT(0:MXN)*60,FILNM*12
****************************************************************************
*       Additional INTEGERS
*       ^^^^^^^^^^^^^^^^^^^
*       NHEAD   - No. of header lines in input file
*       REGNO   - Number of region as read from file
****************************************************************************
      INTEGER NHEAD,REGNO,I
****************************************************************************
*       Additional CHARACTERS
*       ^^^^^^^^^^^^^^^^^^^^^
*       TXT     - Region text as read from file
****************************************************************************
      CHARACTER TXT*60

      NHEAD = 19
      NREGS = -1

      DO 10 I=1,NHEAD
       READ(FILNO,*)
 10   CONTINUE
      
 20   READ(FILNO,FMT=1,ERR=96,END=90) REGNO,TXT
*
*       Check area numbers are consecutive
*
      IF (REGNO.NE.NREGS+1) CALL READERR(FILNM,NREGS+2+NHEAD,11)
*
*       Successful read - increment number of regions by 1 &
*       put into array
*
      NREGS = NREGS+1
      REGTXT(NREGS) = TXT
      GOTO 20

 90   CLOSE(FILNO)
      IF (NREGS.LT.0) CALL READERR(FILNM,0,10)
      RETURN
*
*       Error trapping
*
 96   CALL READERR(FILNM,NHEAD+2+NREGS,2)

 1    FORMAT(I5,A60)
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE CHKREG(REGTXT,NREGS,MXN)
*
*       Checks definitions of regions and writes to output file
*
      INTEGER NREGS,MXN
      CHARACTER REGTXT(0:MXN)*60
      INTEGER I
      
      WRITE(*,'(/''SUBAREA DEFINITIONS'',/19(''=''),/)')
      WRITE(60,'(/''SUBAREA DEFINITIONS'',/19(''=''),/)')

      WRITE(*,'(''REGION: '',A60,/)') REGTXT(0)
      WRITE(60,'(''REGION: '',A60,/)') REGTXT(0)

      IF (NREGS.EQ.0) THEN
       WRITE(*,'(''No subareas defined.'')')
       WRITE(60,'(''No subareas defined.'')')
      ELSEIF (NREGS.GT.0) THEN
       DO 10 I=1,NREGS
	WRITE(*,'(''Subarea '',I2,'': '',A60)') I,REGTXT(I)
	WRITE(60,'(''Subarea '',I2,'': '',A60)') I,REGTXT(I)
 10    CONTINUE
      ELSE
       WRITE(*,1)
      ENDIF

      CALL OKCONT

 1    FORMAT('****ERROR**** No. of subareas < 0! Please inform REC')
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE CHKSIT(REGCODES,SCODES,SITETXT,NSITES,MXN,MXP)
*
*       Checks definitions of sites and writes to output file
*
      INTEGER REGCODES(MXN),NSITES,MXN,MXP
      CHARACTER SCODES(MXN)*4,SITETXT(MXN)*70

      INTEGER I
      
      WRITE(*,'(/''SITE DEFINITIONS'',/16(''=''),/)')
      WRITE(60,'(/''SITE DEFINITIONS'',/16(''=''),/)')

      WRITE(*,1)
      WRITE(60,1)
      
      IF (NSITES.GT.0) THEN
       DO 10 I=1,NSITES
	IF (REGCODES(I).NE.0) THEN
	 WRITE(*,'(I4,2X,A4,2X,I6,2X,A50)') I,SCODES(I),REGCODES(I),
     +                                                  SITETXT(I)
	 WRITE(60,'(I4,2X,A4,2X,I6,2X,A50)') I,SCODES(I),REGCODES(I),
     +                                                  SITETXT(I)
	ELSE
	 WRITE(*,'(I4,2X,A4,10X,A50)') I,SCODES(I),SITETXT(I)
	 WRITE(60,'(I4,2X,A4,10X,A50)') I,SCODES(I),SITETXT(I)
	ENDIF
 10    CONTINUE
      ELSE
       WRITE(*,2)
      ENDIF

      CALL OKCONT

 1    FORMAT('Site',2X,'Site',2X,'Region',2X,'Site',/,
     +       ' no.',2X,'code',2X,' code ',2X,'name',/,
     +       '----',2X,'----',2X,'------',2X,'----',/)
 2    FORMAT(/'****ERROR**** No sites defined! Please inform REC')
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE ANSET(ANSMEAN,ANSVAR,NU,NSITES,MXN,MXP)
*
*       Takes correlation matrix of Anscombe residuals and converts
*	to covariance matrix, based on shape parameter NU of the
*	gamma distributions. ANSMEAN is vector of mean Anscombe 
*	residuals (output), ANSVAR the correlation (input) and
*	covariance (output) matrix, NU the shape parameter (input),
*	NSITES the number of sites, and MXN and MXP are used for 
*	dimensioning.
******************************************************************************
      INTEGER NSITES,MXN,MXP
      DOUBLE PRECISION ANSMEAN(MXN),ANSVAR(MXN,MXN),NU
******************************************************************************
*       Additional DOUBLES
*       ^^^^^^^^^^^^^^^^^^
*       MEAN    - mean Anscombe residual
*       VAR     - Variance of Anscombe residuals
******************************************************************************
      DOUBLE PRECISION MEAN,VAR
******************************************************************************
*       Additional INTEGERs
*       ^^^^^^^^^^^^^^^^^^^
*       I,J	- counters
******************************************************************************
      INTEGER I,J
*
*       First find mean and variance of Anscombe residuals
*
      CALL ANSPRM(NU,MEAN,VAR)
*
*	Now calculate the covariance matrix
*
      DO 100 I=1,NSITES
       ANSMEAN(I) = MEAN
       ANSVAR(I,I) = VAR
       DO 105 J=I+1,NSITES
	ANSVAR(I,J) = ANSVAR(I,J) * VAR
        ANSVAR(J,I) = ANSVAR(I,J)
 105   CONTINUE
 100  CONTINUE

      RETURN
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE MSCSET(SEED)
*
*       To set miscellaneous features. Currently these are:
*       SEED    - seed for random number generator. 
*
      INTEGER SEED

      WRITE(*,'(/''MISCELLANEA'',/11(''=''),/)')
      WRITE(60,'(/''MISCELLANEA'',/11(''=''),/)')

 30   WRITE(*,2)
      READ(*,*,ERR=30) SEED
      IF (SEED.LE.0) THEN
       WRITE(*,'(''****ERROR**** Seed must be positive. Try again.'')')
       GOTO 30
      ENDIF
      WRITE(60,5) SEED
      CALL ZBQLINI(SEED)

 2    FORMAT(/,'Input seed for random number generator (integer < ',
     +'424967291):')
 5    FORMAT('Seed for random number generator: ',I10)
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE SIMSET(YRLIMS,MOLIMS,NSIMS,OUTOPT,OUTFNO,
     +                REGTXT,NREGS,WHICHREGS,OUTSIM,OUTYR,OUTMO,MXN)
*
*       Sets start and end years/months, number of simulations and
*       output options
*
      INTEGER YRLIMS(2),MOLIMS(2),NSIMS,OUTOPT,OUTFNO,MXN
      INTEGER NREGS,WHICHREGS(0:MXN)
      INTEGER OUTSIM,OUTYR(2),OUTMO(2)
      CHARACTER*60 REGTXT(0:MXN)
******************************************************************************
*       Additional INTEGERS
*       ^^^^^^^^^^^^^^^^^^^
*       TOGGLE  - used to toggle a region's inclusion on or off
******************************************************************************
      INTEGER TOGGLE,I
******************************************************************************
*       Additional CHARACTERs
*       ^^^^^^^^^^^^^^^^^^^^^
*       YESNO   - used to print 'Y' or 'N' depending on whether a
*                 region is included in the analysis
******************************************************************************
      CHARACTER YESNO(0:1)*1

      DATA (YESNO(I),I=0,1) /'N','Y'/

      WRITE(*,'(/3(70(''*''),/))')
      WRITE(OUTFNO,'(/3(70(''*''),/))')

      WRITE(*,'(/''SIMULATION OPTIONS'',/18(''=''),/)')
      WRITE(OUTFNO,'(/''SIMULATION OPTIONS'',/18(''=''),/)')
*
*       Read ...
*
 20   WRITE(*,*) 'Input start year and month (YYYYMM):'
      READ(*,FMT='(I4,I2)',ERR=20) YRLIMS(1),MOLIMS(1)
      IF( (YRLIMS(1).LT.1900).OR.(MOLIMS(1).LE.0) ) THEN
       WRITE(*,'(''Er ... perhaps not. Please have another go!'',/)')
       GOTO 20
      ENDIF

 30   WRITE(*,*) 'Now end year and month:'
      READ(*,FMT='(I4,I2)',ERR=30) YRLIMS(2),MOLIMS(2)
      IF( (YRLIMS(2).LT.1900).OR.(MOLIMS(2).LE.0) ) THEN
       WRITE(*,'(''Er ... perhaps not. Please have another go!'',/)')
       GOTO 30
      ENDIF

      IF ((100*(YRLIMS(2)-YRLIMS(1)))+MOLIMS(2)-MOLIMS(1).LT.0) THEN
       WRITE(*,'(''****ERROR**** End is before start! Try again.'')')
       GOTO 20
      ENDIF
      
 40   WRITE(*,*) 'Now input number of simulations/realisations:'
      READ(*,*,ERR=40) NSIMS

 50   WRITE(*,2) 
      READ(*,*,ERR=50) OUTOPT
      IF (.NOT.( (OUTOPT.GE.1).AND.(OUTOPT.LE.3) )) THEN
       WRITE(*,'(''****ERROR**** Invalid option. Try again.'')')
       GOTO 50
      ENDIF
*
*       ... and write to summary output file
*
      WRITE(OUTFNO,1) (MOLIMS(I),YRLIMS(I),I=1,2),NSIMS
      WRITE(OUTFNO,'(''Output options selected'',/,23(''-''),/)')
      IF ( (OUTOPT.EQ.1).OR.(OUTOPT.EQ.3) ) THEN
       WRITE(OUTFNO,'(''Summary information for these subareas:'',/)')
*
*       If regional summaries are requested, determine regions to
*       include
*
 59    IF (NREGS.GT.0) THEN
	WRITE(*,3)
	DO 60 I=1,NREGS
	 WRITE(*,'(2X,I2,4X,A60,3X,A1)') I,REGTXT(I),
     +                                   YESNO(WHICHREGS(I))
 60     CONTINUE
	READ(*,*,ERR=59) TOGGLE
	IF (TOGGLE.EQ.0) GOTO 69
	IF ( (TOGGLE.GE.1).AND.(TOGGLE.LE.NREGS) ) THEN
	 WHICHREGS(TOGGLE) = 1 - WHICHREGS(TOGGLE)
	ELSE
	 DO 65 I=1,NREGS
	  WHICHREGS(I) = 1
 65      CONTINUE
	ENDIF
        GOTO 59
       ENDIF

 69    DO 70 I=0,NREGS
	IF (WHICHREGS(I).EQ.1) WRITE(60,'(2X,I2,5X,A60)') I,REGTXT(I)
 70    CONTINUE

      ENDIF
*
*       And if daily summaries are requested, allow user to specify just
*       a portion of a single realisation
*
      IF ( (OUTOPT.EQ.2).OR.(OUTOPT.EQ.3) ) THEN
 80    WRITE(*,6) 
       READ(*,FMT='(I4,I2)',ERR=80) OUTSIM
       IF (OUTSIM.EQ.0) THEN
	OUTYR(1) = YRLIMS(1)
	OUTYR(2) = YRLIMS(2)
	OUTMO(1) = MOLIMS(1)
	OUTMO(2) = MOLIMS(2)
       ELSEIF ((OUTSIM.GT.0).AND.(OUTSIM.LE.NSIMS)) THEN
 81     WRITE(*,*) 'Input start year and month for daily output ',
     +                          '(YYYYMM):'
	READ(*,FMT='(I4,I2)',ERR=81) OUTYR(1),OUTMO(1)
 82     WRITE(*,*) 'Now end year and month:'
	READ(*,FMT='(I4,I2)',ERR=82) OUTYR(2),OUTMO(2)
	IF (100*(OUTYR(2)-OUTYR(1))+OUTMO(2)-OUTMO(1).LT.0) THEN
	 WRITE(*,7)
	 GOTO 80
	ELSEIF( (100*(OUTYR(2)-YRLIMS(1))+OUTMO(2)-MOLIMS(1).LT.0).OR.
     +        (100*(OUTYR(1)-YRLIMS(2))+OUTMO(1)-MOLIMS(2).GT.0) ) THEN
	 WRITE(*,907)
	 GOTO 80
	ENDIF
       ELSE
	WRITE(*,8) NSIMS
	GOTO 80
       ENDIF         
      ENDIF

      RETURN
 
 1    FORMAT('Simulation runs from month ',I2,'/',I4,
     +' to ',I2,'/',I4,'.',/,I3,' realisations requested.',//)
 2    FORMAT(/,'Now please select an output option:',//5X,
     +'1.  Output regional monthly and annual summary information',
     +/5X,'2.  Output daily values for each site',
     +/5X,'3.  Output both detail and summary information',/)
 3    FORMAT(/,'Choose subareas for inclusion in output file ',
     +'(enter a subarea ',/,'number to toggle on/off, zero to ',
     +'finish and any other number to',/,'include all subareas)',
     +//,'NUMBER  NAME',56X,'INCLUDE?',/,'------  ----',
     +56X,'--------')
 6    FORMAT(/'You have requested daily output for each site defined.',
     +' You may',/,'restrict this output to a portion of one ',
     +'realisation if required.',/,'Please enter 0 now to produce ',
     +'complete output for every realisation.',/,'Otherwise, ',
     +'enter the number of the single realisation for which ',
     +/,'output is required:')
 7    FORMAT(/'****ERROR**** End period precedes start period.',
     +' Try again.')
 8    FORMAT(/'****ERROR**** I''m only doing ',I3,' realisations!',
     +'Please try again.')
 9    FORMAT(/'****ERROR**** Output file ',A12,' already exists')
 907  FORMAT(/'****ERROR**** Outside requested simulation range.'
     +,' Try again.')     
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE UPDAT2(RAIN,PREVIOUS,THRESH,THRTYP,FLAGS,NSITES,
     +               YY,MM,DD,MXLAG,DATINIT,MXN,FILNM,FILNO,SCODES)
*
*       This updates the values held in the RAIN and PREVIOUS arrays. The
*       way in which this is done depends on the value of DATINIT: if
*       equal to 0, a data file is opened if it exists, and the entries are
*       filled in as far as possible from that. If equal to 1 the next
*       day's entry in the data file is read; if equal to 2 then we've got
*       to the end of the file and are on our own. `Soft' and `Hard'
*       thresholding are also done here
*
      INTEGER MXN,DATINIT,FLAGS(MXN),YY,MM,DD
      INTEGER MXLAG,NSITES,THRTYP,FILNO
      DOUBLE PRECISION RAIN(MXN),PREVIOUS(MXN,10),THRESH
      CHARACTER*12 FILNM
      CHARACTER SCODES(MXN)*4
******************************************************************************
*       Additional INTEGERS
*       ^^^^^^^^^^^^^^^^^^^
*       DP      - Previous day 
*       MP      - Month corresponding to previous day
*       DAYS_IN_MONTH   - needed to calculate DP
*       YP      - Year corresponding to previous day
*       NLINES  - No. of lines to skip at beginning of input file
******************************************************************************
      INTEGER DP,MP,YP,DAYS_IN_MONTH
      INTEGER NLINES
      INTEGER I,J

      SAVE NLINES       

*
*	On the first simulation, we're going to have to work out 
*	how many lines of the input file to skip
*
      IF (DATINIT.LT.0) NLINES = 0
*
*       First update the PREVIOUS values. 
*
      DO 10 I=1,NSITES
       DO 11 J=MXLAG,2,-1
        PREVIOUS(I,J) = PREVIOUS(I,J-1)
 11    CONTINUE
       PREVIOUS(I,1) = RAIN(I)
       RAIN(I) = -1.0D0
       FLAGS(I) = 1
 10   CONTINUE

*
*       If DATINIT = 2, we're on our own and that's all we need to do
*
      IF (DATINIT.EQ.2) RETURN
*
*       DATINIT=<0: data not yet initialised. We need to define both
*       RAIN and PREVIOUS, if possible.
*
      IF (DATINIT.LE.0) THEN
       IF (DATINIT.LT.0) THEN
	WRITE(*,1) FILNM
       ELSE
	DO 12 I=1,NLINES
	 READ(FILNO,*)
 12     CONTINUE
       ENDIF
*
*       First define PREVIOUS - we work out what the date was MXLAG days 
*	before today ...
*
       DP = DD - MXLAG
       IF (DP.GT.0) THEN
	MP = MM
	YP = YY
       ELSE
	MP = MM-1
	IF (MP.EQ.0) THEN
	 MP = 12
	 YP = YY-1
	ELSE
	 YP = YY
	ENDIF
	DP = DAYS_IN_MONTH(MP,YP) + DP
       ENDIF
*
*       ... (that last bit was right - DP will be negative!) and read 
*       data for date DP/MP/YP. Array dimensioning makes it necessary to use 
*       RAIN rather than PREVIOUS - we copy it over. On exit from DATRD2,
*       we always check that we haven't got to EOF (a bit pedantic, but
*       some pillock's bound to try a simulation starting on the last day
*       for which data are available! Also reset FLAGS in this case
*
      CALL DATRD2(RAIN,FLAGS,SCODES,NSITES,YP,MP,DP,DATINIT,
     +					MXLAG,NLINES,FILNM,FILNO,MXN)
       DO 15 I = 1,NSITES
*
*	Don't forget - we're reading the value for MXLAG days ago at
*	the moment! Only update the value of PREVIOUS if we found some
*       data in the file.
*
	IF (RAIN(I).GE.-1.0D-12) PREVIOUS(I,MXLAG) = RAIN(I)
	RAIN(I) = -1.0D0
	FLAGS(I) = 1
 15    CONTINUE
       IF (DATINIT.EQ.2) RETURN
*
*       Now up to 1 day before
*
       DO 25 J=MXLAG-1,1,-1
	DP = DP+1
	IF (DP.GT.DAYS_IN_MONTH(MP,YP)) THEN
	 DP = DP+1
	 MP = MP+1
	 IF (MP.GT.12) THEN
	  MP = 1
	  YP = YP + 1
	 ENDIF
	ENDIF
	CALL DATRD2(RAIN,FLAGS,SCODES,NSITES,YP,MP,DP,DATINIT,
     +                                MXLAG,NLINES,FILNM,FILNO,MXN)
	DO 24 I = 1,NSITES
	 IF (RAIN(I).GE.-1.0D-12) PREVIOUS(I,J) = RAIN(I)
	 RAIN(I) = -1.0D0
	 FLAGS(I) = 1
 24     CONTINUE
 25    CONTINUE
       IF (DATINIT.EQ.2) RETURN
*
*       Now it's just as though DATINIT were 1, so we carry on
*
      ENDIF
*
*       Read data for today, and set DATINIT if necessary. 
*    
      CALL DATRD2(RAIN,FLAGS,SCODES,NSITES,YY,MM,DD,DATINIT,
     +                                MXLAG,NLINES,FILNM,FILNO,MXN)
      IF (DATINIT.EQ.0) DATINIT = 1
*
*       Threshold if requested
*
      IF (THRTYP.GT.0) THEN
       DO 20 I = 1,NSITES
        IF (RAIN(I).LT.0.0D0) GOTO 20
        IF (RAIN(I).LT.THRESH) THEN
         RAIN(I) = 0.0D0
        ELSEIF (THRTYP.EQ.1) THEN
         RAIN(I) = RAIN(I) - THRESH
        ENDIF
 20    CONTINUE
      ENDIF

      RETURN

 1    FORMAT(//'Hang on a minute, I''m just searching file ',A12,
     +' for some data within ',/,'the period requested...')
 2    FORMAT(//'****WARNING**** Input file ',A12,' doesn''t exist.',
     +' If you choose to continue, ',/,'it will be as though the ',
     +'time period of your simulation were preceded by 3 completely',
     +/,'dry days.',/)
      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE DATRD2(RAIN,FLAGS,SCODES,NSITES,YY,MM,DD,DATINIT,
     +                                 MXLAG,NLINES,FILNM,FILNO,MXN)
*
*       Reads in rows of data from file connected to unit FILNO and allocates
*       them to array RAIN if they correspond to date DD/MM/YY. Also updates
*       DATINIT if necessary, and counts the number of lines before the first
*       data value if 
*
      INTEGER YY,MM,DD,DATINIT,NLINES,MXN,NSITES
      INTEGER FILNO,FLAGS(MXN),MXLAG
      DOUBLE PRECISION RAIN(MXN)
      CHARACTER*4 SCODES(MXN)
      CHARACTER*12 FILNM
******************************************************************************
*       Additional INTEGERS
*       ^^^^^^^^^^^^^^^^^^^
*       DF      - Day, as read from input file
*       MF      - Month, ditto
*       YF      - Year, ditto
*       IOERR   - Input file error flag - will be < 0 when EOF is attained
******************************************************************************
      INTEGER DF,MF,YF,IOERR,I
******************************************************************************
*       Additional DOUBLEs
*       ^^^^^^^^^^^^^^^^^^
*       RAINF   - rainfall value read from file
******************************************************************************
      DOUBLE PRECISION RAINF
******************************************************************************
*       Additional CHARACTERs
*       ^^^^^^^^^^^^^^^^^^^^^
*       CODEF   - site code as defined in input file
******************************************************************************
      CHARACTER CODEF*4

      YF=0
      MF=0
      DF=0
*
*       Read a line of the input file. If we're at the beginning of the
*       first simulation, keep track of how many lines we've read
*
 19   READ(FILNO,'(I4,I2,I2,A4,F6.2)',ERR=91,END=25,IOSTAT=IOERR)
     +                                      YF,MF,DF,CODEF,RAINF
      IF (DATINIT.LT.0) NLINES = NLINES + 1
*
*       If we've got the right day, update RAIN according to the site code. 
*       If this is the first time through, stop counting lines
*
      IF ( (10000*(YF-YY))+(100*(MF-MM))+DF-DD.EQ.0) THEN
       IF (DATINIT.LT.0) DATINIT = 0
       DO 20 I=1,NSITES
	IF (CODEF.EQ.SCODES(I)) THEN
	 RAIN(I) = RAINF
	 FLAGS(I) = 0
	ENDIF
 20    CONTINUE
*
*       If we overshoot, rewind the input file 1 line and go back. If
*	we overshoot and we haven't yet found any data, issue a 
*	warning.
*
      ELSEIF ((10000*(YF-YY))+(100*(MF-MM))+DF-DD.GT.0) THEN
       BACKSPACE (UNIT=FILNO)
       IF (DATINIT.LT.0) THEN
        DATINIT = 0
        WRITE(*,2) FILNM,MXLAG
        CALL OKCONT
       ENDIF
       RETURN
*
*       If the input file is the wrong order, stop with an error 
*       message. 
*
      ELSE
       IF ((DATINIT.EQ.0).OR.(DATINIT.EQ.1)) GOTO 92
      ENDIF
*
*       Otherwise keep looking
*
      GOTO 19
*
*       Check for end of file -set DATINIT equal to 2 and rewind the file
*	ready for next simulation
*       
 25   IF (IOERR.LT.0) THEN
       DATINIT = 2
       REWIND(FILNO)
      ENDIF
    
      RETURN
*
*       Error trapping
*
 91   WRITE(*,1) FILNM,DF,MF,YF
      STOP
 92   WRITE(*,3) DF,MF,YF,CODEF
      STOP

 1    FORMAT(//'****ERROR**** while reading input file ',A12,'.',
     +' The last successful access',/,'was for data on ',
     +I2,'/',I2,'/',I4,'.')
 2    FORMAT(//'****WARNING**** Your simulation starts before',
     +' the first available data',
     +/'in file ',A12,'. If you continue, it will be as though the',
     +' time period of',
     +/'of your simulation were preceded by ',I1,
     +' totally dry days.',/)
 3    FORMAT(//'****ERROR**** input file out of order.',
     +' Offending date is',/,I2,'/',I2,'/',I4,', at site ',A4,'.')

      END
******************************************************************************
******************************************************************************
******************************************************************************
      SUBROUTINE SETPRED(PRED,COVS,NP,NSITES,COVCODE,TWOWAY,THREEWAY,
     +                             	BETA,PUPDT,MXN,MXP)
*******************************************************************************
*       Calculates linear predictor in a model - updates only the
*	bits requested by the status of PUPDT
*******************************************************************************
*       INTEGER variables
*       ^^^^^^^^^^^^^^^^^
*       NP      - No. of parameters in the model. Input
*       NSITES  - No. of sites. Input
*       COVCODE - Codes for covariates in the model. Input
*       TWOWAY  - list of 2-way interactions. Input
*       THREEWAY- list of 3-way interactions. Input
*	PUPDT	- Indicates which predictors to update. Input/output
*       MONTH   - the current month. Input
*       MXN     - Maximum no. of sites. Input
*       MXP     - Maximum no. of parameters. Input
*******************************************************************************
      INTEGER NP(10),NSITES,COVCODE(MXP),TWOWAY(MXP,2),THREEWAY(MXP,3)
      INTEGER MXN,MXP
      INTEGER PUPDT
      INTEGER I,J
*******************************************************************************
*       DOUBLE PRECISION variables
*       ^^^^^^^^^^^^^^^^^^^^^^^^^^
*       PRED    - site contribution to linear predictor. Output
*       COVS    - Values of covariates at each site
*       BETA    - Vector of parameters
*******************************************************************************
      DOUBLE PRECISION PRED(MXN,4),COVS(MXN,MXP),BETA(0:MXP)

*
*       Site effects ...
*

      IF (PUPDT.LE.1) THEN
       DO 100 I=1,NSITES
        PRED(I,1) = BETA(0)
        DO 105 J=1,NP(1)
 	 PRED(I,1) = PRED(I,1) + (BETA(J)*COVS(I,J))
 105    CONTINUE
 100   CONTINUE
*
*       2-way interactions - only those between site effects
*
       DO 120 J=NP(4)+1,NP(5)
        IF ( (TWOWAY(J,1).LE.NP(1)).AND.(TWOWAY(J,2).LE.NP(1)) ) THEN
	 DO 125 I=1,NSITES
 	  PRED(I,1) = PRED(I,1) + (BETA(J)*COVS(I,J))
 125     CONTINUE
        ENDIF
 120   CONTINUE  
*
*       And 3-way interactions
*
       DO 130 J=NP(5)+1,NP(6)
        IF ( (THREEWAY(J,1).LE.NP(1)).AND.
     +       (THREEWAY(J,2).LE.NP(1)).AND.
     +       (THREEWAY(J,3).LE.NP(1))) THEN
	 DO 135 I=1,NSITES
 	  PRED(I,1) = PRED(I,1) + (BETA(J)*COVS(I,J))
 135     CONTINUE
        ENDIF
 130   CONTINUE  
      ENDIF
*
*       Year effects ...
*

      IF (PUPDT.LE.2) THEN
       DO 250 I=1,NSITES
        PRED(I,2) = PRED(I,1)
 250   CONTINUE
       DO 300 J=NP(1)+1,NP(2)
        DO 305 I=1,NSITES
         PRED(I,2) = PRED(I,2) + (BETA(J)*COVS(I,J))
 305    CONTINUE
 300   CONTINUE
*
*       2-way interactions (year-year and year-site) ...
*
       DO 320 J=NP(4)+1,NP(5)
        IF ( (TWOWAY(J,1).LE.NP(2)).AND.(TWOWAY(J,2).LE.NP(2)) ) THEN
	 IF ( (TWOWAY(J,1).LE.NP(1)).AND.(TWOWAY(J,2).LE.NP(1)) )
     +                                                   GOTO 320
	 DO 325 I=1,NSITES
	  PRED(I,2) = PRED(I,2) + (BETA(J)*COVS(I,J))
 325     CONTINUE
        ENDIF
 320   CONTINUE  
*
*       And 3-way interactions
*
       DO 330 J=NP(5)+1,NP(6)
        IF ( (THREEWAY(J,1).LE.NP(2)).AND.
     +       (THREEWAY(J,2).LE.NP(2)).AND.
     +       (THREEWAY(J,3).LE.NP(2))) THEN
	 IF ( (THREEWAY(J,1).LE.NP(1)).AND.
     +        (THREEWAY(J,2).LE.NP(1)).AND.
     +        (THREEWAY(J,3).LE.NP(1))) GOTO 330
	 DO 335 I=1,NSITES
 	  PRED(I,2) = PRED(I,2) + (BETA(J)*COVS(I,J))
 335     CONTINUE
        ENDIF
 330   CONTINUE  
      ENDIF

*
*	Seasonal effects ...
*
      IF (PUPDT.LE.3) THEN
       DO 450 I=1,NSITES
        PRED(I,3) = PRED(I,2)
 450   CONTINUE
       DO 500 J=NP(2)+1,NP(3)
        DO 505 I=1,NSITES
 	 PRED(I,3) = PRED(I,3) + (BETA(J)*COVS(I,J))
 505    CONTINUE
 500   CONTINUE
*
*       2-way interactions (month-month, month-year and month-site)
*
       DO 520 J=NP(4)+1,NP(5)
        IF ( (TWOWAY(J,1).LE.NP(3)).AND.(TWOWAY(J,2).LE.NP(3)) ) THEN
	 IF ( (TWOWAY(J,1).LE.NP(2)).AND.(TWOWAY(J,2).LE.NP(2)) )
     +                                                   GOTO 520
	 DO 525 I=1,NSITES
 	  PRED(I,3) = PRED(I,3) + (BETA(J)*COVS(I,J))
 525     CONTINUE
        ENDIF
 520   CONTINUE  
*
*       And 3-way interactions
*
       DO 530 J=NP(5)+1,NP(6)
        IF ( (THREEWAY(J,1).LE.NP(3)).AND.
     +       (THREEWAY(J,2).LE.NP(3)).AND.
     +       (THREEWAY(J,3).LE.NP(3))) THEN
	 IF ( (THREEWAY(J,1).LE.NP(2)).AND.
     +        (THREEWAY(J,2).LE.NP(2)).AND.
     +        (THREEWAY(J,3).LE.NP(2))) GOTO 530
	 DO 535 I=1,NSITES
 	  PRED(I,3) = PRED(I,3) + (BETA(J)*COVS(I,J))
 535     CONTINUE
        ENDIF
 530   CONTINUE  
      ENDIF
*
*	Previous day - always done so don't need to condition on PUPDT.
*
      DO 650 I=1,NSITES
       PRED(I,4) = PRED(I,3)
 650  CONTINUE
      DO 700 J=NP(3)+1,NP(4)
       DO 705 I=1,NSITES
	PRED(I,4) = PRED(I,4) + (BETA(J)*COVS(I,J))
 705   CONTINUE
 700  CONTINUE
*
*       2-way interactions ...
*
      DO 720 J=NP(4)+1,NP(5)
       IF ( (TWOWAY(J,1).LE.NP(4)).AND.(TWOWAY(J,2).LE.NP(4)) ) THEN
	IF ( (TWOWAY(J,1).LE.NP(3)).AND.(TWOWAY(J,2).LE.NP(3)) )
     +                                                   GOTO 720
	DO 725 I=1,NSITES
	 PRED(I,4) = PRED(I,4) + (BETA(J)*COVS(I,J))
 725    CONTINUE
       ENDIF
 720  CONTINUE  
*
*       And 3-way interactions
*
      DO 730 J=NP(5)+1,NP(6)
       IF ( (THREEWAY(J,1).LE.NP(4)).AND.
     +      (THREEWAY(J,2).LE.NP(4)).AND.
     +      (THREEWAY(J,3).LE.NP(4))) THEN
	IF ( (THREEWAY(J,1).LE.NP(3)).AND.
     +       (THREEWAY(J,2).LE.NP(3)).AND.
     +       (THREEWAY(J,3).LE.NP(3))) GOTO 730
	DO 735 I=1,NSITES
	 PRED(I,4) = PRED(I,4) + (BETA(J)*COVS(I,J))
 735    CONTINUE
       ENDIF
 730  CONTINUE  

      END
*******************************************************************************
*******************************************************************************
*******************************************************************************
      SUBROUTINE OUTSUM(SIMNO,YEAR,YRTOT,MOTOT,NSITES,REGCODE,
     +                                          WHICHREGS,NREGS,MXN)
*
*       Calculates annual and monthly summary statistics from the site stats
*       contained in YRTOT and MOTOT, and writes records in the summary output
*       file for each region selected in WHICHREGS. NREGS is the total no.
*       of regions defined
*
******************************************************************************
*       INTEGER variables
*       ^^^^^^^^^^^^^^^^^
*       MXNSITES - maximum possible number of sites. Used for dimensioning
*		   internal variables
*	MXN	 - ditto, used for dimensioning arguments.
*       YEAR - current year
*       NSITES - no. of sites
*       REGCODE- indicates which region each site is in
*       WHICHREGS - indicates which regions we want summary measures for
******************************************************************************
      INTEGER MXNSITES
      PARAMETER (MXNSITES = 200)

      INTEGER MXN,YEAR,NSITES,REGCODE(MXN),WHICHREGS(0:MXN)
      INTEGER NREGS,SIMNO
      INTEGER I,J
******************************************************************************
*       DOUBLE PRECISION variables
*       ^^^^^^^^^^^^^^^^^^^^^^^^^^
*       YRTOT   - annual rainfall totals for each site
*       MOTOT   - monthly       "    "     "
*       MOMEAN  - monthly areal means } we have an element for each region.
*       YRMEAN  - annual areal means  } The number of regions is < MXNSITES
*       WEIGHTS - weights for averaging in each region
******************************************************************************
      DOUBLE PRECISION MOTOT(MXN,12),YRTOT(MXN)
      DOUBLE PRECISION MOMEAN(0:MXNSITES,12),YRMEAN(0:MXNSITES)
      DOUBLE PRECISION WEIGHTS(0:MXNSITES)
******************************************************************************
*	Check dimensioning:
*
      IF (MXN.NE.MXNSITES) CALL PRGERR(' OUTSUM ','  simrain.f ',1)
*
*       Initialize ...
*
      DO 100 I = 0,NSITES
       YRMEAN(I) = 0.0D0
       WEIGHTS(I) = 0.0D0
       DO 101 J=1,12
	MOMEAN(I,J) = 0.0D0
 101   CONTINUE
 100  CONTINUE

*
*       and calculate totals and weights for each region. This is a little bit
*       cunning ... for each site, the element of the ??MEAN array corresponding
*       to the region code is updated. The WEIGHTS for this region are only
*       incremented if this region has been chosen. If no region is defined
*       for a site (i.e. REGCODE is 0) we just update the whole area stats
*
      DO 150 I=1,NSITES
       IF (REGCODE(I).NE.0) THEN
	YRMEAN(REGCODE(I)) = YRMEAN(REGCODE(I)) + YRTOT(I)
	WEIGHTS(REGCODE(I)) = WEIGHTS(REGCODE(I))
     +                  + (1.0D0*DBLE(WHICHREGS(REGCODE(I))))
       ENDIF
       YRMEAN(0) = YRMEAN(0) + YRTOT(I)
       WEIGHTS(0) = WEIGHTS(0) + (1.0D0*DBLE(WHICHREGS(0)))
       DO 155 J=1,12
	IF (REGCODE(I).NE.0) THEN
	 MOMEAN(REGCODE(I),J) = MOMEAN(REGCODE(I),J) + MOTOT(I,J)
	ENDIF
	MOMEAN(0,J) = MOMEAN(0,J) + MOTOT(I,J)
 155   CONTINUE
 150  CONTINUE
*
*       Now write a record for each region with non-zero weights. 
*
      DO 200 I=0,NREGS
       IF (WEIGHTS(I).LE.0.0D0) GOTO 200
       DO 205 J=1,12
	MOMEAN(I,J) = MOMEAN(I,J)/WEIGHTS(I)
 205   CONTINUE
       YRMEAN(I) = YRMEAN(I)/WEIGHTS(I)
       WRITE(51,1) SIMNO,YEAR,I,(MOMEAN(I,J),J=1,12),YRMEAN(I)
 200  CONTINUE

 1    FORMAT(I3,1X,I4,1X,I3,1X,12(F6.1,1X),F6.1)
      END

